from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PropertyType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PROPERTY_TYPE_UNSET: _ClassVar[PropertyType]
    PROPERTY_TYPE_INT32: _ClassVar[PropertyType]
    PROPERTY_TYPE_UINT32: _ClassVar[PropertyType]
    PROPERTY_TYPE_INT64: _ClassVar[PropertyType]
    PROPERTY_TYPE_UINT64: _ClassVar[PropertyType]
    PROPERTY_TYPE_FLOAT: _ClassVar[PropertyType]
    PROPERTY_TYPE_DOUBLE: _ClassVar[PropertyType]
    PROPERTY_TYPE_STRING: _ClassVar[PropertyType]
    PROPERTY_TYPE_DATETIME: _ClassVar[PropertyType]
    PROPERTY_TYPE_TIMESTAMP: _ClassVar[PropertyType]
    PROPERTY_TYPE_TEXT: _ClassVar[PropertyType]
    PROPERTY_TYPE_BLOB: _ClassVar[PropertyType]
    PROPERTY_TYPE_POINT: _ClassVar[PropertyType]
    PROPERTY_TYPE_DECIMAL: _ClassVar[PropertyType]
    PROPERTY_TYPE_LIST: _ClassVar[PropertyType]
    PROPERTY_TYPE_SET: _ClassVar[PropertyType]
    PROPERTY_TYPE_MAP: _ClassVar[PropertyType]
    PROPERTY_TYPE_NULL: _ClassVar[PropertyType]
    PROPERTY_TYPE_BOOL: _ClassVar[PropertyType]
    PROPERTY_TYPE_LOCAL_DATETIME: _ClassVar[PropertyType]
    PROPERTY_TYPE_ZONED_DATETIME: _ClassVar[PropertyType]
    PROPERTY_TYPE_DATE: _ClassVar[PropertyType]
    PROPERTY_TYPE_ZONED_TIME: _ClassVar[PropertyType]
    PROPERTY_TYPE_LOCAL_TIME: _ClassVar[PropertyType]
    PROPERTY_TYPE_YEAR_TO_MONTH: _ClassVar[PropertyType]
    PROPERTY_TYPE_DAY_TO_SECOND: _ClassVar[PropertyType]
    PROPERTY_TYPE_RECORD: _ClassVar[PropertyType]
    PROPERTY_TYPE_POINT3D: _ClassVar[PropertyType]
    PROPERTY_TYPE_VECTOR: _ClassVar[PropertyType]
    PROPERTY_TYPE_TABLE: _ClassVar[PropertyType]
    PROPERTY_TYPE_PATH: _ClassVar[PropertyType]
    PROPERTY_TYPE_ERROR: _ClassVar[PropertyType]
    PROPERTY_TYPE_NODE: _ClassVar[PropertyType]
    PROPERTY_TYPE_EDGE: _ClassVar[PropertyType]

class GraphType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    GRAPH_TYPE_OPEN: _ClassVar[GraphType]
    GRAPH_TYPE_CLOSED: _ClassVar[GraphType]
    GRAPH_TYPE_ONTOLOGY: _ClassVar[GraphType]

class HealthStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    HEALTH_STATUS_UNKNOWN: _ClassVar[HealthStatus]
    HEALTH_STATUS_SERVING: _ClassVar[HealthStatus]
    HEALTH_STATUS_NOT_SERVING: _ClassVar[HealthStatus]
    HEALTH_STATUS_SERVICE_UNKNOWN: _ClassVar[HealthStatus]

class CacheType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    CACHE_TYPE_ALL: _ClassVar[CacheType]
    CACHE_TYPE_AST: _ClassVar[CacheType]
    CACHE_TYPE_PLAN: _ClassVar[CacheType]
PROPERTY_TYPE_UNSET: PropertyType
PROPERTY_TYPE_INT32: PropertyType
PROPERTY_TYPE_UINT32: PropertyType
PROPERTY_TYPE_INT64: PropertyType
PROPERTY_TYPE_UINT64: PropertyType
PROPERTY_TYPE_FLOAT: PropertyType
PROPERTY_TYPE_DOUBLE: PropertyType
PROPERTY_TYPE_STRING: PropertyType
PROPERTY_TYPE_DATETIME: PropertyType
PROPERTY_TYPE_TIMESTAMP: PropertyType
PROPERTY_TYPE_TEXT: PropertyType
PROPERTY_TYPE_BLOB: PropertyType
PROPERTY_TYPE_POINT: PropertyType
PROPERTY_TYPE_DECIMAL: PropertyType
PROPERTY_TYPE_LIST: PropertyType
PROPERTY_TYPE_SET: PropertyType
PROPERTY_TYPE_MAP: PropertyType
PROPERTY_TYPE_NULL: PropertyType
PROPERTY_TYPE_BOOL: PropertyType
PROPERTY_TYPE_LOCAL_DATETIME: PropertyType
PROPERTY_TYPE_ZONED_DATETIME: PropertyType
PROPERTY_TYPE_DATE: PropertyType
PROPERTY_TYPE_ZONED_TIME: PropertyType
PROPERTY_TYPE_LOCAL_TIME: PropertyType
PROPERTY_TYPE_YEAR_TO_MONTH: PropertyType
PROPERTY_TYPE_DAY_TO_SECOND: PropertyType
PROPERTY_TYPE_RECORD: PropertyType
PROPERTY_TYPE_POINT3D: PropertyType
PROPERTY_TYPE_VECTOR: PropertyType
PROPERTY_TYPE_TABLE: PropertyType
PROPERTY_TYPE_PATH: PropertyType
PROPERTY_TYPE_ERROR: PropertyType
PROPERTY_TYPE_NODE: PropertyType
PROPERTY_TYPE_EDGE: PropertyType
GRAPH_TYPE_OPEN: GraphType
GRAPH_TYPE_CLOSED: GraphType
GRAPH_TYPE_ONTOLOGY: GraphType
HEALTH_STATUS_UNKNOWN: HealthStatus
HEALTH_STATUS_SERVING: HealthStatus
HEALTH_STATUS_NOT_SERVING: HealthStatus
HEALTH_STATUS_SERVICE_UNKNOWN: HealthStatus
CACHE_TYPE_ALL: CacheType
CACHE_TYPE_AST: CacheType
CACHE_TYPE_PLAN: CacheType

class TypedValue(_message.Message):
    __slots__ = ("type", "data", "is_null")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    IS_NULL_FIELD_NUMBER: _ClassVar[int]
    type: PropertyType
    data: bytes
    is_null: bool
    def __init__(self, type: _Optional[_Union[PropertyType, str]] = ..., data: _Optional[bytes] = ..., is_null: bool = ...) -> None: ...

class Parameter(_message.Message):
    __slots__ = ("name", "value")
    NAME_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    name: str
    value: TypedValue
    def __init__(self, name: _Optional[str] = ..., value: _Optional[_Union[TypedValue, _Mapping]] = ...) -> None: ...

class Row(_message.Message):
    __slots__ = ("values",)
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedCompositeFieldContainer[TypedValue]
    def __init__(self, values: _Optional[_Iterable[_Union[TypedValue, _Mapping]]] = ...) -> None: ...

class LoginRequest(_message.Message):
    __slots__ = ("username", "password", "default_graph")
    USERNAME_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_GRAPH_FIELD_NUMBER: _ClassVar[int]
    username: str
    password: str
    default_graph: str
    def __init__(self, username: _Optional[str] = ..., password: _Optional[str] = ..., default_graph: _Optional[str] = ...) -> None: ...

class LoginResponse(_message.Message):
    __slots__ = ("session_id", "server_version", "roles")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    SERVER_VERSION_FIELD_NUMBER: _ClassVar[int]
    ROLES_FIELD_NUMBER: _ClassVar[int]
    session_id: int
    server_version: str
    roles: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, session_id: _Optional[int] = ..., server_version: _Optional[str] = ..., roles: _Optional[_Iterable[str]] = ...) -> None: ...

class LogoutRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class LogoutResponse(_message.Message):
    __slots__ = ("success",)
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    def __init__(self, success: bool = ...) -> None: ...

class PingRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class PingResponse(_message.Message):
    __slots__ = ("latency_ns",)
    LATENCY_NS_FIELD_NUMBER: _ClassVar[int]
    latency_ns: int
    def __init__(self, latency_ns: _Optional[int] = ...) -> None: ...

class GqlRequest(_message.Message):
    __slots__ = ("gql", "graph_name", "parameters", "session_id", "transaction_id", "timeout", "read_only", "max_path_results")
    GQL_FIELD_NUMBER: _ClassVar[int]
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    MAX_PATH_RESULTS_FIELD_NUMBER: _ClassVar[int]
    gql: str
    graph_name: str
    parameters: _containers.RepeatedCompositeFieldContainer[Parameter]
    session_id: int
    transaction_id: int
    timeout: int
    read_only: bool
    max_path_results: int
    def __init__(self, gql: _Optional[str] = ..., graph_name: _Optional[str] = ..., parameters: _Optional[_Iterable[_Union[Parameter, _Mapping]]] = ..., session_id: _Optional[int] = ..., transaction_id: _Optional[int] = ..., timeout: _Optional[int] = ..., read_only: bool = ..., max_path_results: _Optional[int] = ...) -> None: ...

class GqlResponse(_message.Message):
    __slots__ = ("columns", "rows", "row_count", "has_more", "warnings", "rows_affected")
    COLUMNS_FIELD_NUMBER: _ClassVar[int]
    ROWS_FIELD_NUMBER: _ClassVar[int]
    ROW_COUNT_FIELD_NUMBER: _ClassVar[int]
    HAS_MORE_FIELD_NUMBER: _ClassVar[int]
    WARNINGS_FIELD_NUMBER: _ClassVar[int]
    ROWS_AFFECTED_FIELD_NUMBER: _ClassVar[int]
    columns: _containers.RepeatedScalarFieldContainer[str]
    rows: _containers.RepeatedCompositeFieldContainer[Row]
    row_count: int
    has_more: bool
    warnings: _containers.RepeatedScalarFieldContainer[str]
    rows_affected: int
    def __init__(self, columns: _Optional[_Iterable[str]] = ..., rows: _Optional[_Iterable[_Union[Row, _Mapping]]] = ..., row_count: _Optional[int] = ..., has_more: bool = ..., warnings: _Optional[_Iterable[str]] = ..., rows_affected: _Optional[int] = ...) -> None: ...

class ExplainResponse(_message.Message):
    __slots__ = ("plan",)
    PLAN_FIELD_NUMBER: _ClassVar[int]
    plan: str
    def __init__(self, plan: _Optional[str] = ...) -> None: ...

class ProfileResponse(_message.Message):
    __slots__ = ("profile",)
    PROFILE_FIELD_NUMBER: _ClassVar[int]
    profile: str
    def __init__(self, profile: _Optional[str] = ...) -> None: ...

class NodeData(_message.Message):
    __slots__ = ("id", "labels", "properties")
    class PropertiesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: TypedValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[TypedValue, _Mapping]] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    LABELS_FIELD_NUMBER: _ClassVar[int]
    PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    id: str
    labels: _containers.RepeatedScalarFieldContainer[str]
    properties: _containers.MessageMap[str, TypedValue]
    def __init__(self, id: _Optional[str] = ..., labels: _Optional[_Iterable[str]] = ..., properties: _Optional[_Mapping[str, TypedValue]] = ...) -> None: ...

class EdgeData(_message.Message):
    __slots__ = ("label", "from_node_id", "to_node_id", "properties")
    class PropertiesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: TypedValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[TypedValue, _Mapping]] = ...) -> None: ...
    LABEL_FIELD_NUMBER: _ClassVar[int]
    FROM_NODE_ID_FIELD_NUMBER: _ClassVar[int]
    TO_NODE_ID_FIELD_NUMBER: _ClassVar[int]
    PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    label: str
    from_node_id: str
    to_node_id: str
    properties: _containers.MessageMap[str, TypedValue]
    def __init__(self, label: _Optional[str] = ..., from_node_id: _Optional[str] = ..., to_node_id: _Optional[str] = ..., properties: _Optional[_Mapping[str, TypedValue]] = ...) -> None: ...

class BulkCreateNodesOptions(_message.Message):
    __slots__ = ("overwrite",)
    OVERWRITE_FIELD_NUMBER: _ClassVar[int]
    overwrite: bool
    def __init__(self, overwrite: bool = ...) -> None: ...

class BulkCreateEdgesOptions(_message.Message):
    __slots__ = ("skip_invalid_nodes",)
    SKIP_INVALID_NODES_FIELD_NUMBER: _ClassVar[int]
    skip_invalid_nodes: bool
    def __init__(self, skip_invalid_nodes: bool = ...) -> None: ...

class InsertNodesRequest(_message.Message):
    __slots__ = ("graph_name", "nodes", "options", "bulk_import_session_id")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    NODES_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    BULK_IMPORT_SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    nodes: _containers.RepeatedCompositeFieldContainer[NodeData]
    options: BulkCreateNodesOptions
    bulk_import_session_id: str
    def __init__(self, graph_name: _Optional[str] = ..., nodes: _Optional[_Iterable[_Union[NodeData, _Mapping]]] = ..., options: _Optional[_Union[BulkCreateNodesOptions, _Mapping]] = ..., bulk_import_session_id: _Optional[str] = ...) -> None: ...

class InsertNodesResponse(_message.Message):
    __slots__ = ("success", "node_ids", "node_count", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    NODE_IDS_FIELD_NUMBER: _ClassVar[int]
    NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    node_ids: _containers.RepeatedScalarFieldContainer[str]
    node_count: int
    message: str
    def __init__(self, success: bool = ..., node_ids: _Optional[_Iterable[str]] = ..., node_count: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class InsertEdgesRequest(_message.Message):
    __slots__ = ("graph_name", "edges", "options", "bulk_import_session_id")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    EDGES_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    BULK_IMPORT_SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    edges: _containers.RepeatedCompositeFieldContainer[EdgeData]
    options: BulkCreateEdgesOptions
    bulk_import_session_id: str
    def __init__(self, graph_name: _Optional[str] = ..., edges: _Optional[_Iterable[_Union[EdgeData, _Mapping]]] = ..., options: _Optional[_Union[BulkCreateEdgesOptions, _Mapping]] = ..., bulk_import_session_id: _Optional[str] = ...) -> None: ...

class InsertEdgesResponse(_message.Message):
    __slots__ = ("success", "edge_ids", "edge_count", "message", "skipped_count")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    EDGE_IDS_FIELD_NUMBER: _ClassVar[int]
    EDGE_COUNT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    SKIPPED_COUNT_FIELD_NUMBER: _ClassVar[int]
    success: bool
    edge_ids: _containers.RepeatedScalarFieldContainer[str]
    edge_count: int
    message: str
    skipped_count: int
    def __init__(self, success: bool = ..., edge_ids: _Optional[_Iterable[str]] = ..., edge_count: _Optional[int] = ..., message: _Optional[str] = ..., skipped_count: _Optional[int] = ...) -> None: ...

class DeleteNodesRequest(_message.Message):
    __slots__ = ("graph_name", "node_ids", "labels", "where")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    NODE_IDS_FIELD_NUMBER: _ClassVar[int]
    LABELS_FIELD_NUMBER: _ClassVar[int]
    WHERE_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    node_ids: _containers.RepeatedScalarFieldContainer[str]
    labels: _containers.RepeatedScalarFieldContainer[str]
    where: str
    def __init__(self, graph_name: _Optional[str] = ..., node_ids: _Optional[_Iterable[str]] = ..., labels: _Optional[_Iterable[str]] = ..., where: _Optional[str] = ...) -> None: ...

class DeleteNodesResponse(_message.Message):
    __slots__ = ("success", "deleted_count", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    DELETED_COUNT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    deleted_count: int
    message: str
    def __init__(self, success: bool = ..., deleted_count: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class DeleteEdgesRequest(_message.Message):
    __slots__ = ("graph_name", "edge_ids", "label", "where")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    EDGE_IDS_FIELD_NUMBER: _ClassVar[int]
    LABEL_FIELD_NUMBER: _ClassVar[int]
    WHERE_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    edge_ids: _containers.RepeatedScalarFieldContainer[str]
    label: str
    where: str
    def __init__(self, graph_name: _Optional[str] = ..., edge_ids: _Optional[_Iterable[str]] = ..., label: _Optional[str] = ..., where: _Optional[str] = ...) -> None: ...

class DeleteEdgesResponse(_message.Message):
    __slots__ = ("success", "deleted_count", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    DELETED_COUNT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    deleted_count: int
    message: str
    def __init__(self, success: bool = ..., deleted_count: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class ExportRequest(_message.Message):
    __slots__ = ("graph_name", "batch_size", "export_nodes", "export_edges", "node_labels", "edge_labels", "include_metadata")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    BATCH_SIZE_FIELD_NUMBER: _ClassVar[int]
    EXPORT_NODES_FIELD_NUMBER: _ClassVar[int]
    EXPORT_EDGES_FIELD_NUMBER: _ClassVar[int]
    NODE_LABELS_FIELD_NUMBER: _ClassVar[int]
    EDGE_LABELS_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_METADATA_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    batch_size: int
    export_nodes: bool
    export_edges: bool
    node_labels: _containers.RepeatedScalarFieldContainer[str]
    edge_labels: _containers.RepeatedScalarFieldContainer[str]
    include_metadata: bool
    def __init__(self, graph_name: _Optional[str] = ..., batch_size: _Optional[int] = ..., export_nodes: bool = ..., export_edges: bool = ..., node_labels: _Optional[_Iterable[str]] = ..., edge_labels: _Optional[_Iterable[str]] = ..., include_metadata: bool = ...) -> None: ...

class ExportResponse(_message.Message):
    __slots__ = ("data", "is_final", "stats")
    DATA_FIELD_NUMBER: _ClassVar[int]
    IS_FINAL_FIELD_NUMBER: _ClassVar[int]
    STATS_FIELD_NUMBER: _ClassVar[int]
    data: bytes
    is_final: bool
    stats: ExportStats
    def __init__(self, data: _Optional[bytes] = ..., is_final: bool = ..., stats: _Optional[_Union[ExportStats, _Mapping]] = ...) -> None: ...

class ExportStats(_message.Message):
    __slots__ = ("nodes_exported", "edges_exported", "bytes_written", "duration_ms")
    NODES_EXPORTED_FIELD_NUMBER: _ClassVar[int]
    EDGES_EXPORTED_FIELD_NUMBER: _ClassVar[int]
    BYTES_WRITTEN_FIELD_NUMBER: _ClassVar[int]
    DURATION_MS_FIELD_NUMBER: _ClassVar[int]
    nodes_exported: int
    edges_exported: int
    bytes_written: int
    duration_ms: int
    def __init__(self, nodes_exported: _Optional[int] = ..., edges_exported: _Optional[int] = ..., bytes_written: _Optional[int] = ..., duration_ms: _Optional[int] = ...) -> None: ...

class CreateGraphRequest(_message.Message):
    __slots__ = ("name", "graph_type", "description")
    NAME_FIELD_NUMBER: _ClassVar[int]
    GRAPH_TYPE_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    name: str
    graph_type: GraphType
    description: str
    def __init__(self, name: _Optional[str] = ..., graph_type: _Optional[_Union[GraphType, str]] = ..., description: _Optional[str] = ...) -> None: ...

class CreateGraphResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class DropGraphRequest(_message.Message):
    __slots__ = ("name", "if_exists")
    NAME_FIELD_NUMBER: _ClassVar[int]
    IF_EXISTS_FIELD_NUMBER: _ClassVar[int]
    name: str
    if_exists: bool
    def __init__(self, name: _Optional[str] = ..., if_exists: bool = ...) -> None: ...

class DropGraphResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class UseGraphRequest(_message.Message):
    __slots__ = ("name", "session_id")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    name: str
    session_id: int
    def __init__(self, name: _Optional[str] = ..., session_id: _Optional[int] = ...) -> None: ...

class UseGraphResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class ListGraphsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListGraphsResponse(_message.Message):
    __slots__ = ("graphs",)
    GRAPHS_FIELD_NUMBER: _ClassVar[int]
    graphs: _containers.RepeatedCompositeFieldContainer[GraphInfo]
    def __init__(self, graphs: _Optional[_Iterable[_Union[GraphInfo, _Mapping]]] = ...) -> None: ...

class GetGraphInfoRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class GetGraphInfoResponse(_message.Message):
    __slots__ = ("info",)
    INFO_FIELD_NUMBER: _ClassVar[int]
    info: GraphInfo
    def __init__(self, info: _Optional[_Union[GraphInfo, _Mapping]] = ...) -> None: ...

class GraphInfo(_message.Message):
    __slots__ = ("name", "graph_type", "node_count", "edge_count", "description")
    NAME_FIELD_NUMBER: _ClassVar[int]
    GRAPH_TYPE_FIELD_NUMBER: _ClassVar[int]
    NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    EDGE_COUNT_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    name: str
    graph_type: GraphType
    node_count: int
    edge_count: int
    description: str
    def __init__(self, name: _Optional[str] = ..., graph_type: _Optional[_Union[GraphType, str]] = ..., node_count: _Optional[int] = ..., edge_count: _Optional[int] = ..., description: _Optional[str] = ...) -> None: ...

class BeginRequest(_message.Message):
    __slots__ = ("session_id", "graph_name", "read_only", "timeout")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    session_id: int
    graph_name: str
    read_only: bool
    timeout: int
    def __init__(self, session_id: _Optional[int] = ..., graph_name: _Optional[str] = ..., read_only: bool = ..., timeout: _Optional[int] = ...) -> None: ...

class BeginResponse(_message.Message):
    __slots__ = ("transaction_id", "message")
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    transaction_id: int
    message: str
    def __init__(self, transaction_id: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class CommitRequest(_message.Message):
    __slots__ = ("session_id", "transaction_id")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: int
    transaction_id: int
    def __init__(self, session_id: _Optional[int] = ..., transaction_id: _Optional[int] = ...) -> None: ...

class CommitResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class RollbackRequest(_message.Message):
    __slots__ = ("session_id", "transaction_id")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: int
    transaction_id: int
    def __init__(self, session_id: _Optional[int] = ..., transaction_id: _Optional[int] = ...) -> None: ...

class RollbackResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class ListTransactionsRequest(_message.Message):
    __slots__ = ("session_id", "all_transactions")
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    ALL_TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    session_id: int
    all_transactions: bool
    def __init__(self, session_id: _Optional[int] = ..., all_transactions: bool = ...) -> None: ...

class ListTransactionsResponse(_message.Message):
    __slots__ = ("transactions",)
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    transactions: _containers.RepeatedCompositeFieldContainer[TransactionInfo]
    def __init__(self, transactions: _Optional[_Iterable[_Union[TransactionInfo, _Mapping]]] = ...) -> None: ...

class TransactionInfo(_message.Message):
    __slots__ = ("transaction_id", "session_id", "graph_name", "read_only", "created_at", "duration_ms", "internal_tx_id")
    TRANSACTION_ID_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    DURATION_MS_FIELD_NUMBER: _ClassVar[int]
    INTERNAL_TX_ID_FIELD_NUMBER: _ClassVar[int]
    transaction_id: int
    session_id: int
    graph_name: str
    read_only: bool
    created_at: int
    duration_ms: int
    internal_tx_id: str
    def __init__(self, transaction_id: _Optional[int] = ..., session_id: _Optional[int] = ..., graph_name: _Optional[str] = ..., read_only: bool = ..., created_at: _Optional[int] = ..., duration_ms: _Optional[int] = ..., internal_tx_id: _Optional[str] = ...) -> None: ...

class HealthCheckRequest(_message.Message):
    __slots__ = ("service",)
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    service: str
    def __init__(self, service: _Optional[str] = ...) -> None: ...

class HealthCheckResponse(_message.Message):
    __slots__ = ("status",)
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: HealthStatus
    def __init__(self, status: _Optional[_Union[HealthStatus, str]] = ...) -> None: ...

class WarmupParserRequest(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class WarmupParserResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class GetCacheStatsRequest(_message.Message):
    __slots__ = ("cache_type",)
    CACHE_TYPE_FIELD_NUMBER: _ClassVar[int]
    cache_type: CacheType
    def __init__(self, cache_type: _Optional[_Union[CacheType, str]] = ...) -> None: ...

class CacheStatsResponse(_message.Message):
    __slots__ = ("ast_stats", "plan_stats")
    AST_STATS_FIELD_NUMBER: _ClassVar[int]
    PLAN_STATS_FIELD_NUMBER: _ClassVar[int]
    ast_stats: ASTCacheStats
    plan_stats: PlanCacheStats
    def __init__(self, ast_stats: _Optional[_Union[ASTCacheStats, _Mapping]] = ..., plan_stats: _Optional[_Union[PlanCacheStats, _Mapping]] = ...) -> None: ...

class ASTCacheStats(_message.Message):
    __slots__ = ("hits", "misses", "evictions", "entries", "hit_rate")
    HITS_FIELD_NUMBER: _ClassVar[int]
    MISSES_FIELD_NUMBER: _ClassVar[int]
    EVICTIONS_FIELD_NUMBER: _ClassVar[int]
    ENTRIES_FIELD_NUMBER: _ClassVar[int]
    HIT_RATE_FIELD_NUMBER: _ClassVar[int]
    hits: int
    misses: int
    evictions: int
    entries: int
    hit_rate: float
    def __init__(self, hits: _Optional[int] = ..., misses: _Optional[int] = ..., evictions: _Optional[int] = ..., entries: _Optional[int] = ..., hit_rate: _Optional[float] = ...) -> None: ...

class PlanCacheStats(_message.Message):
    __slots__ = ("size", "capacity", "hits", "misses", "hit_rate")
    SIZE_FIELD_NUMBER: _ClassVar[int]
    CAPACITY_FIELD_NUMBER: _ClassVar[int]
    HITS_FIELD_NUMBER: _ClassVar[int]
    MISSES_FIELD_NUMBER: _ClassVar[int]
    HIT_RATE_FIELD_NUMBER: _ClassVar[int]
    size: int
    capacity: int
    hits: int
    misses: int
    hit_rate: float
    def __init__(self, size: _Optional[int] = ..., capacity: _Optional[int] = ..., hits: _Optional[int] = ..., misses: _Optional[int] = ..., hit_rate: _Optional[float] = ...) -> None: ...

class ClearCacheRequest(_message.Message):
    __slots__ = ("cache_type",)
    CACHE_TYPE_FIELD_NUMBER: _ClassVar[int]
    cache_type: CacheType
    def __init__(self, cache_type: _Optional[_Union[CacheType, str]] = ...) -> None: ...

class ClearCacheResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class GetStatisticsRequest(_message.Message):
    __slots__ = ("graph_name",)
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    def __init__(self, graph_name: _Optional[str] = ...) -> None: ...

class GetStatisticsResponse(_message.Message):
    __slots__ = ("node_count", "edge_count", "label_counts", "edge_label_counts")
    class LabelCountsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: int
        def __init__(self, key: _Optional[str] = ..., value: _Optional[int] = ...) -> None: ...
    class EdgeLabelCountsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: int
        def __init__(self, key: _Optional[str] = ..., value: _Optional[int] = ...) -> None: ...
    NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    EDGE_COUNT_FIELD_NUMBER: _ClassVar[int]
    LABEL_COUNTS_FIELD_NUMBER: _ClassVar[int]
    EDGE_LABEL_COUNTS_FIELD_NUMBER: _ClassVar[int]
    node_count: int
    edge_count: int
    label_counts: _containers.ScalarMap[str, int]
    edge_label_counts: _containers.ScalarMap[str, int]
    def __init__(self, node_count: _Optional[int] = ..., edge_count: _Optional[int] = ..., label_counts: _Optional[_Mapping[str, int]] = ..., edge_label_counts: _Optional[_Mapping[str, int]] = ...) -> None: ...

class InvalidatePermissionCacheRequest(_message.Message):
    __slots__ = ("username",)
    USERNAME_FIELD_NUMBER: _ClassVar[int]
    username: str
    def __init__(self, username: _Optional[str] = ...) -> None: ...

class InvalidatePermissionCacheResponse(_message.Message):
    __slots__ = ("success",)
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    def __init__(self, success: bool = ...) -> None: ...

class CompactRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class CompactResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class WaitForComputeTopologyRequest(_message.Message):
    __slots__ = ("graph_name", "timeout_ms")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_MS_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    timeout_ms: int
    def __init__(self, graph_name: _Optional[str] = ..., timeout_ms: _Optional[int] = ...) -> None: ...

class WaitForComputeTopologyResponse(_message.Message):
    __slots__ = ("ready", "message")
    READY_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ready: bool
    message: str
    def __init__(self, ready: bool = ..., message: _Optional[str] = ...) -> None: ...

class StartBulkImportRequest(_message.Message):
    __slots__ = ("graph_name", "checkpoint_every", "estimated_nodes", "estimated_edges", "memtable_size", "max_memtables")
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    CHECKPOINT_EVERY_FIELD_NUMBER: _ClassVar[int]
    ESTIMATED_NODES_FIELD_NUMBER: _ClassVar[int]
    ESTIMATED_EDGES_FIELD_NUMBER: _ClassVar[int]
    MEMTABLE_SIZE_FIELD_NUMBER: _ClassVar[int]
    MAX_MEMTABLES_FIELD_NUMBER: _ClassVar[int]
    graph_name: str
    checkpoint_every: int
    estimated_nodes: int
    estimated_edges: int
    memtable_size: int
    max_memtables: int
    def __init__(self, graph_name: _Optional[str] = ..., checkpoint_every: _Optional[int] = ..., estimated_nodes: _Optional[int] = ..., estimated_edges: _Optional[int] = ..., memtable_size: _Optional[int] = ..., max_memtables: _Optional[int] = ...) -> None: ...

class StartBulkImportResponse(_message.Message):
    __slots__ = ("success", "session_id", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    session_id: str
    message: str
    def __init__(self, success: bool = ..., session_id: _Optional[str] = ..., message: _Optional[str] = ...) -> None: ...

class CheckpointRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class CheckpointResponse(_message.Message):
    __slots__ = ("success", "record_count", "last_checkpoint_count", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    RECORD_COUNT_FIELD_NUMBER: _ClassVar[int]
    LAST_CHECKPOINT_COUNT_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    record_count: int
    last_checkpoint_count: int
    message: str
    def __init__(self, success: bool = ..., record_count: _Optional[int] = ..., last_checkpoint_count: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class EndBulkImportRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class EndBulkImportResponse(_message.Message):
    __slots__ = ("success", "total_records", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    TOTAL_RECORDS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    total_records: int
    message: str
    def __init__(self, success: bool = ..., total_records: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class AbortBulkImportRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class AbortBulkImportResponse(_message.Message):
    __slots__ = ("success", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    def __init__(self, success: bool = ..., message: _Optional[str] = ...) -> None: ...

class GetBulkImportStatusRequest(_message.Message):
    __slots__ = ("session_id",)
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    def __init__(self, session_id: _Optional[str] = ...) -> None: ...

class GetBulkImportStatusResponse(_message.Message):
    __slots__ = ("is_active", "graph_name", "record_count", "last_checkpoint_count", "created_at", "last_activity")
    IS_ACTIVE_FIELD_NUMBER: _ClassVar[int]
    GRAPH_NAME_FIELD_NUMBER: _ClassVar[int]
    RECORD_COUNT_FIELD_NUMBER: _ClassVar[int]
    LAST_CHECKPOINT_COUNT_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    LAST_ACTIVITY_FIELD_NUMBER: _ClassVar[int]
    is_active: bool
    graph_name: str
    record_count: int
    last_checkpoint_count: int
    created_at: int
    last_activity: int
    def __init__(self, is_active: bool = ..., graph_name: _Optional[str] = ..., record_count: _Optional[int] = ..., last_checkpoint_count: _Optional[int] = ..., created_at: _Optional[int] = ..., last_activity: _Optional[int] = ...) -> None: ...



"""Type definitions for GQLDB Python driver.

This file re-exports all types from the types/ subdirectory for backward compatibility.
"""

# Re-export everything from types module
from .types import *  # noqa: F401, F403



"""Data Service integration tests."""

import pytest
import uuid

from gqldb.client import GqldbClient, QueryConfig
from gqldb.response import ExportConfig, ExportChunk
from gqldb.types import GraphType, NodeData, EdgeData
from tests.conftest import drop_test_graph


@pytest.mark.integration
class TestData:
    """Tests for data service (insert/export nodes and edges)."""

    @pytest.fixture
    def test_graph(self, connected_client: GqldbClient, request):
        """Create a temporary test graph."""
        graph_name = f"test_{request.node.name}_{uuid.uuid4().hex[:8]}"
        connected_client.create_graph(graph_name, GraphType.OPEN, "Test data graph")
        yield graph_name
        drop_test_graph(connected_client, graph_name)

    def test_insert_nodes(self, connected_client: GqldbClient, test_graph: str):
        """Test inserting nodes."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            nodes = [
                NodeData(labels=["Person"], properties={"name": "Alice", "age": 30}),
                NodeData(labels=["Person"], properties={"name": "Bob", "age": 25}),
            ]

            result = connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )

            assert result.success
            assert result.node_count == 2
            print(f"Inserted {result.node_count} nodes")
        finally:
            connected_client.end_bulk_import(session.session_id)

    def test_insert_edges(self, connected_client: GqldbClient, test_graph: str):
        """Test inserting edges."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            # First insert nodes
            nodes = [
                NodeData(labels=["Person"], properties={"name": "Alice"}),
                NodeData(labels=["Person"], properties={"name": "Bob"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )

            # Query to get node IDs (bulk import doesn't return IDs immediately)
            query_config = QueryConfig(graph_name=test_graph)
            resp = connected_client.gql("MATCH (n:Person) RETURN id(n) AS node_id ORDER BY n.name", query_config)

            node_ids = []
            for i in range(resp.row_count):
                row = resp.rows[i]
                value = resp.get_by_name(row, "node_id")
                if value:
                    node_ids.append(value)

            # Then insert edges
            if len(node_ids) >= 2:
                edges = [
                    EdgeData(
                        label="KNOWS",
                        from_node_id=node_ids[0],
                        to_node_id=node_ids[1],
                        properties={"since": 2020}
                    ),
                ]
                result = connected_client.insert_edges(
                    test_graph,
                    edges,
                    bulk_import_session_id=session.session_id
                )

                assert result.success
                print(f"Inserted {result.edge_count} edges")
        finally:
            connected_client.end_bulk_import(session.session_id)

    def test_export(self, connected_client: GqldbClient, test_graph: str):
        """Test unified export with ExportConfig."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            # First insert some nodes
            nodes = [
                NodeData(labels=["Person"], properties={"name": "Charlie"}),
                NodeData(labels=["Person"], properties={"name": "Diana"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )

        finally:
            connected_client.end_bulk_import(session.session_id)

        # Export using new unified API
        chunks = []
        all_data = []

        def callback(chunk: ExportChunk):
            chunks.append(chunk)
            all_data.append(chunk.get_data_as_string())
            if chunk.is_final and chunk.has_stats():
                print(f"Export stats: nodes={chunk.stats.nodes_exported}, edges={chunk.stats.edges_exported}")

        config = ExportConfig(
            graph_name=test_graph,
            export_nodes=True,
            export_edges=True,
            batch_size=1000
        )
        connected_client.export(config, callback)
        print(f"Total chunks: {len(chunks)}, data length: {sum(len(d) for d in all_data)} bytes")

    def test_export_nodes_only(self, connected_client: GqldbClient, test_graph: str):
        """Test export with nodes only."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            nodes = [
                NodeData(labels=["Person"], properties={"name": "ExportNode1"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )
        finally:
            connected_client.end_bulk_import(session.session_id)

        chunks = []

        def callback(chunk: ExportChunk):
            chunks.append(chunk)

        config = ExportConfig(
            graph_name=test_graph,
            export_nodes=True,
            export_edges=False
        )
        connected_client.export(config, callback)
        print(f"Export nodes-only: {len(chunks)} chunks")

    def test_export_edges_only(self, connected_client: GqldbClient, test_graph: str):
        """Test export with edges only."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            nodes = [
                NodeData(labels=["Person"], properties={"name": "EdgeNode1"}),
                NodeData(labels=["Person"], properties={"name": "EdgeNode2"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )
            # Query to get node IDs
            query_config = QueryConfig(graph_name=test_graph)
            resp = connected_client.gql("MATCH (n:Person) WHERE n.name STARTS WITH 'EdgeNode' RETURN id(n) AS node_id ORDER BY n.name", query_config)

            node_ids = []
            for i in range(resp.row_count):
                row = resp.rows[i]
                value = resp.get_by_name(row, "node_id")
                if value:
                    node_ids.append(value)

            if len(node_ids) >= 2:
                edges = [
                    EdgeData(
                        label="EDGE_ONLY_TEST",
                        from_node_id=node_ids[0],
                        to_node_id=node_ids[1],
                        properties={}
                    ),
                ]
                connected_client.insert_edges(
                    test_graph,
                    edges,
                    bulk_import_session_id=session.session_id
                )
        finally:
            connected_client.end_bulk_import(session.session_id)

        chunks = []

        def callback(chunk: ExportChunk):
            chunks.append(chunk)

        config = ExportConfig(
            graph_name=test_graph,
            export_nodes=False,
            export_edges=True
        )
        connected_client.export(config, callback)
        print(f"Export edges-only: {len(chunks)} chunks")

    def test_delete_nodes(self, connected_client: GqldbClient, test_graph: str):
        """Test deleting nodes by ID."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            # Insert nodes first
            nodes = [
                NodeData(labels=["ToDelete"], properties={"name": "ToDelete1"}),
                NodeData(labels=["ToDelete"], properties={"name": "ToDelete2"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )

        finally:
            connected_client.end_bulk_import(session.session_id)

        # Query to get node IDs (bulk import doesn't return IDs immediately)
        query_config = QueryConfig(graph_name=test_graph)
        resp = connected_client.gql("MATCH (n:ToDelete) RETURN id(n) AS node_id ORDER BY n.name", query_config)

        node_ids = []
        for i in range(resp.row_count):
            row = resp.rows[i]
            value = resp.get_by_name(row, "node_id")
            if value:
                node_ids.append(value)

        assert len(node_ids) == 2
        print(f"Inserted nodes: {node_ids}")

        # Delete nodes by ID
        result = connected_client.delete_nodes(
            test_graph,
            node_ids=node_ids
        )
        assert result.success
        print(f"Deleted {result.deleted_count} nodes")

    def test_delete_edges(self, connected_client: GqldbClient, test_graph: str):
        """Test deleting edges by ID."""
        # Start bulk import session
        session = connected_client.start_bulk_import(test_graph)

        try:
            # Insert nodes first
            nodes = [
                NodeData(labels=["Person"], properties={"name": "Source"}),
                NodeData(labels=["Person"], properties={"name": "Target"}),
            ]
            connected_client.insert_nodes(
                test_graph,
                nodes,
                bulk_import_session_id=session.session_id
            )

            # Query to get node IDs
            query_config = QueryConfig(graph_name=test_graph)
            resp = connected_client.gql("MATCH (n:Person) RETURN id(n) AS node_id ORDER BY n.name", query_config)

            node_ids = []
            for i in range(resp.row_count):
                row = resp.rows[i]
                value = resp.get_by_name(row, "node_id")
                if value:
                    node_ids.append(value)

            if len(node_ids) >= 2:
                # Insert edge
                edges = [
                    EdgeData(
                        label="TO_DELETE",
                        from_node_id=node_ids[0],
                        to_node_id=node_ids[1],
                        properties={"temp": True}
                    ),
                ]
                connected_client.insert_edges(
                    test_graph,
                    edges,
                    bulk_import_session_id=session.session_id
                )

        finally:
            connected_client.end_bulk_import(session.session_id)

        # Query to get edge IDs
        edge_resp = connected_client.gql("MATCH ()-[e:TO_DELETE]->() RETURN id(e) AS edge_id", query_config)

        edge_ids = []
        for i in range(edge_resp.row_count):
            row = edge_resp.rows[i]
            value = edge_resp.get_by_name(row, "edge_id")
            if value:
                edge_ids.append(value)

        print(f"Inserted edge: {edge_ids}")

        # Delete edge by ID
        if edge_ids:
            result = connected_client.delete_edges(
                test_graph,
                edge_ids=edge_ids
            )
            assert result.success
            print(f"Deleted {result.deleted_count} edges")



"""Transaction Service integration tests."""

import pytest
import uuid

from gqldb.client import GqldbClient
from gqldb.config import GqldbConfig
from gqldb.types import GraphType
from tests.conftest import check_auth_enabled, drop_test_graph, TEST_USERNAME, TEST_PASSWORD


@pytest.mark.integration
class TestTransaction:
    """Tests for transaction service.

    Note: Transaction operations require a valid session, which means
    authentication must be enabled on the server.
    """

    @pytest.fixture
    def logged_in_client(self, test_config: GqldbConfig):
        """Return a logged-in client, skip if auth disabled."""
        client = GqldbClient(test_config)
        if not check_auth_enabled(client):
            client.close()
            pytest.skip("Transaction service requires authentication to be enabled")
        client.login(TEST_USERNAME, TEST_PASSWORD)
        yield client
        try:
            client.logout()
        except Exception:
            pass
        client.close()

    @pytest.fixture
    def tx_test_graph(self, logged_in_client: GqldbClient, request):
        """Create a temporary test graph for transaction tests."""
        graph_name = f"tx_test_{uuid.uuid4().hex[:8]}"
        logged_in_client.create_graph(graph_name, GraphType.OPEN, "Transaction test graph")
        yield graph_name
        drop_test_graph(logged_in_client, graph_name)

    def test_begin_and_commit_transaction(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test beginning and committing a transaction."""
        # Begin transaction
        tx = logged_in_client.begin_transaction(tx_test_graph, read_only=False)
        assert tx is not None
        assert tx.id > 0
        print(f"Started transaction: id={tx.id}, graph={tx.graph_name}")

        # Commit transaction
        success = logged_in_client.commit(tx.id)
        assert success
        print("Transaction committed successfully")

    def test_begin_and_rollback_transaction(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test beginning and rolling back a transaction."""
        # Begin transaction
        tx = logged_in_client.begin_transaction(tx_test_graph, read_only=False)
        assert tx is not None
        assert tx.id > 0
        print(f"Started transaction: id={tx.id}")

        # Rollback transaction
        success = logged_in_client.rollback(tx.id)
        assert success
        print("Transaction rolled back successfully")

    def test_read_only_transaction(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test read-only transaction."""
        # Begin read-only transaction
        tx = logged_in_client.begin_transaction(tx_test_graph, read_only=True)
        assert tx is not None
        assert tx.id > 0
        assert tx.read_only
        print(f"Started read-only transaction: id={tx.id}")

        # Commit (or rollback) the read-only transaction
        success = logged_in_client.commit(tx.id)
        assert success
        print("Read-only transaction committed successfully")

    def test_list_transactions(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test listing active transactions."""
        # Begin a transaction
        tx = logged_in_client.begin_transaction(tx_test_graph, read_only=False)
        assert tx is not None

        try:
            # List transactions
            transactions = logged_in_client.list_transactions()
            assert isinstance(transactions, list)
            print(f"Found {len(transactions)} active transactions")
            for t in transactions:
                print(f"  - id={t.transaction_id}, graph={t.graph_name}, read_only={t.read_only}")
        finally:
            # Clean up - rollback the transaction
            logged_in_client.rollback(tx.id)

    def test_with_transaction_commit(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test with_transaction helper for auto-commit."""
        executed = []

        def tx_func(tx_id: int):
            assert tx_id > 0
            executed.append(tx_id)
            print(f"Executing in transaction {tx_id}")

        logged_in_client.with_transaction(tx_test_graph, tx_func)
        assert len(executed) == 1
        print("with_transaction completed successfully (committed)")

    def test_with_transaction_rollback_on_error(self, logged_in_client: GqldbClient, tx_test_graph: str):
        """Test with_transaction helper for auto-rollback on error."""

        def tx_func(tx_id: int):
            print(f"Executing in transaction {tx_id}")
            raise ValueError("Simulated error")

        with pytest.raises(ValueError) as exc_info:
            logged_in_client.with_transaction(tx_test_graph, tx_func)
        assert "Simulated error" in str(exc_info.value)
        print("with_transaction rolled back on error as expected")



"""Numeric type wrappers for explicit type specification."""

from dataclasses import dataclass

@dataclass
class Int32:
    """Wrapper for explicit 32-bit signed integer."""
    value: int


@dataclass
class UInt32:
    """Wrapper for explicit 32-bit unsigned integer."""
    value: int


@dataclass
class Float32:
    """Wrapper for explicit 32-bit float."""
    value: float


@dataclass
class UInt64:
    """Wrapper for explicit 64-bit unsigned integer."""
    value: int



"""Main client for GQLDB Python driver."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Callable, Dict, List, Optional

import grpc

from .config import GqldbConfig
from .connection import ConnectionPool
from .errors import EmptyQueryError, GqldbError, GraphNotFoundError
from .response import (
    DeleteResult,
    ExportChunk,
    ExportConfig,
    ExportedEdge,
    ExportedNode,
    ExportEdgesResult,
    ExportNodesResult,
    ExportStats,
    InsertEdgesResult,
    InsertNodesResult,
    Response,
    Row,
)
from .session import Session, SessionManager
from .transaction import Transaction, TransactionManager
from .types import (
    AbortBulkImportResult,
    ASTCacheStats,
    BulkCreateEdgesOptions,
    BulkCreateNodesOptions,
    BulkImportSession,
    BulkImportStatus,
    CacheStats,
    CacheType,
    CheckpointResult,
    CompactResult,
    EdgeData,
    EndBulkImportResult,
    GraphInfo,
    GraphType,
    HealthStatus,
    NodeData,
    PlanCacheStats,
    PropertyType,
    Statistics,
    TransactionInfo,
    TypedValue,
)


@dataclass
class QueryConfig:
    """Configuration for a query."""
    graph_name: str = ""
    parameters: Dict[str, Any] = field(default_factory=dict)
    transaction_id: int = 0
    timeout: int = 0
    read_only: bool = False
    max_path_results: int = 0  # Max paths to return from path queries (0 = unlimited)


class GqldbClient:
    """Main client for interacting with GQLDB."""

    def __init__(self, config: GqldbConfig):
        """Initialize the GQLDB client.

        Args:
            config: Configuration for the client.

        Raises:
            GqldbError: If initialization fails.
        """
        config.validate()
        self._config = config
        self._pool = ConnectionPool(config)
        self._sessions = SessionManager()
        self._tx_manager = TransactionManager()

        # Initialize service context
        from .services import (
            ServiceContext,
            SessionService,
            QueryService,
            GraphService,
            TransactionService,
            DataService,
            HealthService,
            AdminService,
            BulkImportService,
        )

        ctx = ServiceContext(config, self._pool, self._sessions, self._tx_manager)

        # Initialize service instances
        self._session_service = SessionService(ctx)
        self._query_service = QueryService(ctx)
        self._graph_service = GraphService(ctx)
        self._transaction_service = TransactionService(ctx)
        self._data_service = DataService(ctx)
        self._health_service = HealthService(ctx)
        self._admin_service = AdminService(ctx)
        self._bulk_import_service = BulkImportService(ctx)

    def close(self) -> None:
        """Close the client and all connections."""
        if self._sessions.is_logged_in():
            try:
                self.logout()
            except Exception:
                pass
        self._pool.close()

    def __enter__(self) -> "GqldbClient":
        """Context manager entry."""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """Context manager exit."""
        self.close()

    # =========================================================================
    # Session Service
    # =========================================================================

    def login(self, username: str, password: str) -> Session:
        """Authenticate the user and create a session."""
        return self._session_service.login(username, password, self._config.default_graph)

    def logout(self) -> None:
        """Terminate the current session."""
        return self._session_service.logout()

    def ping(self) -> int:
        """Check the connection and return the latency in nanoseconds."""
        return self._session_service.ping()

    # =========================================================================
    # Query Service
    # =========================================================================

    def gql(self, query: str, config: Optional[QueryConfig] = None) -> Response:
        """Execute a GQL query and return the result.

        Args:
            query: GQL query string.
            config: Optional query configuration.

        Returns:
            Query response.

        Raises:
            EmptyQueryError: If query is empty.
            GqldbError: If query fails.
        """
        return self._query_service.gql(query, config)
    def gql_stream(
        self,
        query: str,
        config: Optional[QueryConfig] = None,
        callback: Optional[Callable[[Response], None]] = None
    ) -> None:
        """Execute a GQL query and stream the results.

        Args:
            query: GQL query string.
            config: Optional query configuration.
            callback: Callback function for each response chunk.

        Raises:
            EmptyQueryError: If query is empty.
            GqldbError: If query fails.
        """
        return self._query_service.gql_stream(query, config, callback)

    def explain(self, query: str, config: Optional[QueryConfig] = None) -> str:
        """Return the execution plan for a query.

        Args:
            query: GQL query string.
            config: Optional query configuration.

        Returns:
            Execution plan string.

        Raises:
            GqldbError: If explain fails.
        """
        return self._query_service.explain(query, config)
    def profile(self, query: str, config: Optional[QueryConfig] = None) -> str:
        """Execute a query with profiling and return statistics.

        Args:
            query: GQL query string.
            config: Optional query configuration.

        Returns:
            Profile statistics string.

        Raises:
            GqldbError: If profile fails.
        """
        return self._query_service.profile(query, config)
    # =========================================================================
    # Graph Service
    # =========================================================================

    def create_graph(
        self,
        name: str,
        graph_type: GraphType = GraphType.OPEN,
        description: str = ""
    ) -> None:
        """Create a new graph.

        Args:
            name: Graph name.
            graph_type: Type of graph.
            description: Optional description.

        Raises:
            GqldbError: If creation fails.
        """
        return self._graph_service.create_graph(name, graph_type, description)
    def drop_graph(self, name: str, if_exists: bool = False) -> None:
        """Delete a graph.

        Args:
            name: Graph name.
            if_exists: If True, don't error if graph doesn't exist.

        Raises:
            GqldbError: If deletion fails.
        """
        return self._graph_service.drop_graph(name, if_exists)
    def use_graph(self, name: str) -> None:
        """Set the current graph for the session.

        Args:
            name: Graph name.

        Raises:
            GqldbError: If operation fails.
        """
        return self._graph_service.use_graph(name)
    def list_graphs(self) -> List[GraphInfo]:
        """Return all available graphs.

        Returns:
            List of GraphInfo objects.

        Raises:
            GqldbError: If operation fails.
        """
        return self._graph_service.list_graphs()
    def get_graph_info(self, name: str) -> GraphInfo:
        """Return information about a specific graph.

        Args:
            name: Graph name.

        Returns:
            GraphInfo object.

        Raises:
            GraphNotFoundError: If graph not found.
            GqldbError: If operation fails.
        """
        return self._graph_service.get_graph_info(name)
    # =========================================================================
    # Transaction Service
    # =========================================================================

    def begin_transaction(
        self,
        graph_name: str,
        read_only: bool = False,
        timeout: int = 0
    ) -> Transaction:
        """Start a new transaction.

        Args:
            graph_name: Graph name for the transaction.
            read_only: Whether the transaction is read-only.
            timeout: Timeout in seconds.

        Returns:
            The created Transaction.

        Raises:
            GqldbError: If operation fails.
        """
        return self._transaction_service.begin_transaction(graph_name, read_only, timeout)
    def commit(self, transaction_id: int) -> bool:
        """Commit a transaction.

        Args:
            transaction_id: Transaction ID.

        Returns:
            True if successful.

        Raises:
            GqldbError: If operation fails.
        """
        return self._transaction_service.commit(transaction_id)
    def rollback(self, transaction_id: int) -> bool:
        """Rollback a transaction.

        Args:
            transaction_id: Transaction ID.

        Returns:
            True if successful.

        Raises:
            GqldbError: If operation fails.
        """
        return self._transaction_service.rollback(transaction_id)
    def list_transactions(self) -> List[TransactionInfo]:
        """Return active transactions.

        Returns:
            List of TransactionInfo objects.

        Raises:
            GqldbError: If operation fails.
        """
        return self._transaction_service.list_transactions()
    def with_transaction(
        self,
        graph_name: str,
        fn: Callable[[int], None],
        read_only: bool = False,
        timeout: Optional[int] = None
    ) -> None:
        """Execute a function within a transaction.

        Automatically commits on success or rolls back on error.

        Args:
            graph_name: Graph name for the transaction.
            fn: Function to execute with transaction ID.
            read_only: Whether the transaction is read-only.
            timeout: Optional timeout in seconds. Uses config timeout if not specified.

        Raises:
            GqldbError: If operation fails.
        """
        return self._transaction_service.with_transaction(
            graph_name, fn, read_only, timeout or self.config.timeout
        )
    # =========================================================================
    # Data Service
    # =========================================================================

    def insert_nodes(
        self,
        graph_name: str,
        nodes: List[NodeData],
        options: Optional[BulkCreateNodesOptions] = None,
        bulk_import_session_id: str = ""
    ) -> InsertNodesResult:
        """Insert multiple nodes into a graph.

        Args:
            graph_name: Graph name.
            nodes: List of node data.
            options: Optional bulk creation options.
            bulk_import_session_id: Optional bulk import session ID for auto-checkpoint.

        Returns:
            InsertNodesResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.insert_nodes(graph_name, nodes, options, bulk_import_session_id)
    def insert_edges(
        self,
        graph_name: str,
        edges: List[EdgeData],
        options: Optional[BulkCreateEdgesOptions] = None,
        bulk_import_session_id: str = ""
    ) -> InsertEdgesResult:
        """Insert multiple edges into a graph.

        Args:
            graph_name: Graph name.
            edges: List of edge data.
            options: Optional bulk creation options.
            bulk_import_session_id: Optional bulk import session ID for auto-checkpoint.

        Returns:
            InsertEdgesResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.insert_edges(graph_name, edges, options, bulk_import_session_id)
    def delete_nodes(
        self,
        graph_name: str,
        node_ids: Optional[List[str]] = None,
        labels: Optional[List[str]] = None,
        where: str = ""
    ) -> DeleteResult:
        """Delete nodes from a graph.

        Args:
            graph_name: Graph name.
            node_ids: Optional list of node IDs to delete.
            labels: Optional list of labels to filter by.
            where: Optional WHERE clause.

        Returns:
            DeleteResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.delete_nodes(graph_name, node_ids, labels, where)
    def delete_edges(
        self,
        graph_name: str,
        edge_ids: Optional[List[str]] = None,
        label: str = "",
        where: str = ""
    ) -> DeleteResult:
        """Delete edges from a graph.

        Args:
            graph_name: Graph name.
            edge_ids: Optional list of edge IDs to delete.
            label: Optional label to filter by.
            where: Optional WHERE clause.

        Returns:
            DeleteResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.delete_edges(graph_name, edge_ids, label, where)
    def export(
        self,
        config: ExportConfig,
        callback: Optional[Callable[[ExportChunk], None]] = None
    ) -> None:
        """Export graph data in JSON Lines format (streaming).

        Args:
            config: Export configuration.
            callback: Callback for each exported chunk.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.export(config, callback)

    def export_nodes(
        self,
        graph_name: str,
        labels: Optional[List[str]] = None,
        limit: int = 0,
        callback: Optional[Callable[[ExportNodesResult], None]] = None
    ) -> None:
        """Stream nodes from a graph.

        Deprecated: Use export() with ExportConfig instead.

        Args:
            graph_name: Graph name.
            labels: Optional list of labels to filter by.
            limit: Maximum number of nodes (0 = no limit).
            callback: Callback for each batch.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.export_nodes(graph_name, labels, limit, callback)

    def export_edges(
        self,
        graph_name: str,
        labels: Optional[List[str]] = None,
        limit: int = 0,
        callback: Optional[Callable[[ExportEdgesResult], None]] = None
    ) -> None:
        """Stream edges from a graph.

        Deprecated: Use export() with ExportConfig instead.

        Args:
            graph_name: Graph name.
            labels: Optional list of labels to filter by.
            limit: Maximum number of edges (0 = no limit).
            callback: Callback for each batch.

        Raises:
            GqldbError: If operation fails.
        """
        return self._data_service.export_edges(graph_name, labels, limit, callback)
    # =========================================================================
    # Health Service
    # =========================================================================

    def health_check(self, service: str = "") -> HealthStatus:
        """Check the health of a service.

        Args:
            service: Service name (empty for overall health).

        Returns:
            HealthStatus.

        Raises:
            GqldbError: If operation fails.
        """
        return self._health_service.health_check(service)
    def _proto_to_health_status(self, proto_status: int) -> HealthStatus:
        """Convert proto health status to HealthStatus enum."""
        # Proto enum values: HEALTH_STATUS_UNKNOWN=0, SERVING=1, NOT_SERVING=2, SERVICE_UNKNOWN=3
        if proto_status == 1:
            return HealthStatus.SERVING
        elif proto_status == 2:
            return HealthStatus.NOT_SERVING
        elif proto_status == 3:
            return HealthStatus.SERVICE_UNKNOWN
        return HealthStatus.UNKNOWN

    def watch(
        self,
        service: str = "",
        callback: Optional[Callable[[HealthStatus], None]] = None
    ) -> None:
        """Watch health status changes via server-side streaming.

        Args:
            service: Service name to watch (empty for overall health).
            callback: Callback function called with each HealthStatus update.

        Raises:
            GqldbError: If operation fails.

        Example:
            >>> def on_status(status: HealthStatus):
            ...     print(f"Health status: {status}")
            >>> client.watch(callback=on_status)
        """
        return self._health_service.watch(service, callback)
    def watch_iter(self, service: str = ""):
        """Watch health status changes and yield HealthStatus values.

        This is a generator version of watch() for iteration.

        Args:
            service: Service name to watch (empty for overall health).

        Yields:
            HealthStatus: Each health status update from the server.

        Example:
            >>> for status in client.watch_iter():
            ...     print(f"Health status: {status}")
            ...     if status != HealthStatus.SERVING:
            ...         break
        """
        return self._health_service.watch_iter(service)
    # =========================================================================
    # Admin Service
    # =========================================================================

    def warmup_parser(self, count: int) -> None:
        """Pre-allocate parser instances.

        Args:
            count: Number of parser instances.

        Raises:
            GqldbError: If operation fails.
        """
        return self._admin_service.warmup_parser(count)
    def get_cache_stats(
        self,
        cache_type: CacheType = CacheType.ALL
    ) -> CacheStats:
        """Return cache statistics.

        Args:
            cache_type: Type of cache to query.

        Returns:
            CacheStats.

        Raises:
            GqldbError: If operation fails.
        """
        return self._admin_service.get_cache_stats(cache_type)
    def clear_cache(self, cache_type: CacheType = CacheType.ALL) -> None:
        """Clear the specified cache.

        Args:
            cache_type: Type of cache to clear.

        Raises:
            GqldbError: If operation fails.
        """
        return self._admin_service.clear_cache(cache_type)
    def get_statistics(self, graph_name: str = "") -> Statistics:
        """Return database statistics.

        Args:
            graph_name: Optional graph name (empty for current graph).

        Returns:
            Statistics.

        Raises:
            GqldbError: If operation fails.
        """
        return self._admin_service.get_statistics(graph_name)
    def invalidate_permission_cache(self, username: str = "") -> None:
        """Invalidate the RBAC permission cache.

        Args:
            username: Optional username (empty for all users).

        Raises:
            GqldbError: If operation fails.
        """
        return self._admin_service.invalidate_permission_cache(username)
    def compact(self) -> CompactResult:
        """Trigger manual compaction of the database storage.

        Returns:
            CompactResult.

        Raises:
            GqldbError: If operation fails.
        """
        from .proto import gqldb_pb2

        if self._admin_stub is None:
            channel = self._get_channel()
            self._init_stubs(channel)

        try:
            request = gqldb_pb2.CompactRequest()
            metadata = self._get_session_metadata()
            response = self._admin_stub.Compact(
                request,
                metadata=metadata,
                timeout=self._config.timeout
            )
            return CompactResult(
                success=response.success,
                message=response.message
            )
        except grpc.RpcError as e:
            raise GqldbError(f"Compact failed: {e.details()}") from e

    # =========================================================================
    # Bulk Import Service
    # =========================================================================

    def start_bulk_import(
        self,
        graph_name: str,
        checkpoint_every: int = 0,
        estimated_nodes: int = 0,
        estimated_edges: int = 0,
        memtable_size: int = 0,
        max_memtables: int = 0
    ) -> BulkImportSession:
        """Start a bulk import session for optimized high-throughput inserts.

        Args:
            graph_name: Graph name.
            checkpoint_every: Deprecated. No longer used, server ignores this value.
            estimated_nodes: Hint for pre-allocating node ID cache.
            estimated_edges: Hint for edge batch sizing.
            memtable_size: Memtable size in bytes before flush (default: 64MB).
            max_memtables: Max immutable memtables before stall (default: 4).

        Returns:
            BulkImportSession.

        Raises:
            GqldbError: If operation fails.
        """
        return self._bulk_import_service.start_bulk_import(graph_name, checkpoint_every, estimated_nodes, estimated_edges, memtable_size, max_memtables)
    def checkpoint(self, session_id: str) -> CheckpointResult:
        """Flush all accumulated data to disk for durability.

        .. deprecated::
            Checkpoint is no longer needed. Use end_bulk_import() which performs a final flush.

        Args:
            session_id: Bulk import session ID.

        Returns:
            CheckpointResult.
        """
        import warnings
        warnings.warn(
            "checkpoint() is deprecated. Use end_bulk_import() which performs a final flush.",
            DeprecationWarning,
            stacklevel=2,
        )
        return CheckpointResult(
            success=True,
            record_count=0,
            last_checkpoint_count=0,
            message="Checkpoint has been removed; use end_bulk_import which performs a final flush",
        )
    def end_bulk_import(self, session_id: str) -> EndBulkImportResult:
        """End the bulk import session with a final checkpoint.

        Args:
            session_id: Bulk import session ID.

        Returns:
            EndBulkImportResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._bulk_import_service.end_bulk_import(session_id)
    def abort_bulk_import(self, session_id: str) -> AbortBulkImportResult:
        """Cancel the bulk import session without final sync.

        Args:
            session_id: Bulk import session ID.

        Returns:
            AbortBulkImportResult.

        Raises:
            GqldbError: If operation fails.
        """
        return self._bulk_import_service.abort_bulk_import(session_id)
    def get_bulk_import_status(self, session_id: str) -> BulkImportStatus:
        """Return the current status of a bulk import session.

        Args:
            session_id: Bulk import session ID.

        Returns:
            BulkImportStatus.

        Raises:
            GqldbError: If operation fails.
        """
        return self._bulk_import_service.get_bulk_import_status(session_id)
    # =========================================================================
    # Convenience Methods
    # =========================================================================

    def get_session(self) -> Optional[Session]:
        """Return the current session."""
        return self._sessions.get_session()

    def is_logged_in(self) -> bool:
        """Check if there is an active session."""
        return self._sessions.is_logged_in()

    @property
    def config(self) -> GqldbConfig:
        """Return the client configuration."""
        return self._config



"""Query Service integration tests."""

import pytest

from gqldb.client import GqldbClient, QueryConfig


@pytest.mark.integration
class TestQuery:
    """Tests for GQL query execution."""

    def test_simple_query(self, connected_client: GqldbClient):
        """Test a simple GQL query."""
        response = connected_client.gql("RETURN 1 AS num")
        assert response is not None
        print(f"Query returned {len(response.rows)} rows")
        print(f"Columns: {response.columns}")

    def test_return_literal(self, connected_client: GqldbClient):
        """Test returning literal values."""
        response = connected_client.gql("RETURN 'hello' AS greeting, 42 AS answer")
        print(f"Query returned {len(response.rows)} rows")
        print(f"Columns: {response.columns}")
        if response.rows:
            for i, row in enumerate(response.rows):
                print(f"Row {i}: {[row.get(j) for j in range(len(response.columns))]}")

    def test_empty_query_raises_error(self, connected_client: GqldbClient):
        """Test that empty query raises EmptyQueryError."""
        from gqldb.errors import EmptyQueryError

        with pytest.raises(EmptyQueryError):
            connected_client.gql("")

    def test_gql_statement(self, connected_client: GqldbClient):
        """Test executing a GQL statement."""
        result = connected_client.gql("RETURN 1")
        assert result is not None
        print(f"Gql result: rows={result.row_count}, columns={result.columns}")

    def test_gql_stream(self, connected_client: GqldbClient):
        """Test streaming GQL query."""
        results = []

        def callback(response):
            results.append(response)
            print(f"Stream chunk: {len(response.rows)} rows")

        connected_client.gql_stream("RETURN 1 AS num", callback=callback)
        print(f"Received {len(results)} stream chunks")

    def test_gql_with_graph_name(self, connected_client: GqldbClient):
        """Test query with explicit graph name."""
        config = QueryConfig(graph_name="miniCircle")
        response = connected_client.gql("MATCH (n) RETURN n LIMIT 5", config)
        assert response is not None
        print(f"Query on miniCircle returned {len(response.rows)} rows")

    def test_explain(self, connected_client: GqldbClient):
        """Test EXPLAIN query."""
        try:
            plan = connected_client.explain("RETURN 1 AS num")
            print(f"Explain plan: {plan}")
        except Exception as e:
            print(f"Explain error: {e}")

    def test_profile(self, connected_client: GqldbClient):
        """Test PROFILE query."""
        try:
            profile = connected_client.profile("RETURN 1 AS num")
            print(f"Profile result: {profile}")
        except Exception as e:
            print(f"Profile error: {e}")


@pytest.mark.integration
class TestMaxPathResults:
    """Tests for max_path_results parameter."""

    def test_query_with_max_path_results(self, connected_client: GqldbClient):
        """Test query with max_path_results limit."""
        try:
            # Try a path query
            config = QueryConfig(max_path_results=5)
            response = connected_client.gql(
                "MATCH p=(n)-[*1..2]-(m) RETURN p LIMIT 10",
                config
            )
            print(f"Path query with max_path_results=5: {len(response.rows)} rows")
        except Exception as e:
            # Path query syntax may not be supported, try simple query
            print(f"Path query failed: {e}, trying simple query")
            config = QueryConfig(max_path_results=5)
            response = connected_client.gql("RETURN 1 AS result", config)
            assert response is not None
            print(f"Simple query with max_path_results=5: {len(response.rows)} rows")

    def test_query_with_max_path_results_zero(self, connected_client: GqldbClient):
        """Test query with max_path_results=0 (unlimited)."""
        config = QueryConfig(max_path_results=0)
        response = connected_client.gql("RETURN 1 AS result", config)
        assert response is not None
        print(f"Query with max_path_results=0 (unlimited): {len(response.rows)} rows")

    def test_query_with_max_path_results_large(self, connected_client: GqldbClient):
        """Test query with large max_path_results value."""
        config = QueryConfig(max_path_results=1000000)
        response = connected_client.gql("RETURN 1 AS result", config)
        assert response is not None
        print(f"Query with max_path_results=1000000: {len(response.rows)} rows")

    def test_gql_stream_with_max_path_results(self, connected_client: GqldbClient):
        """Test streaming query with max_path_results."""
        results = []

        def callback(response):
            results.append(response)
            print(f"Stream chunk with max_path_results=10: {len(response.rows)} rows")

        config = QueryConfig(max_path_results=10)
        connected_client.gql_stream("RETURN 1 AS num", config, callback=callback)
        print(f"Received {len(results)} stream chunks")

    def test_explain_with_max_path_results(self, connected_client: GqldbClient):
        """Test EXPLAIN with max_path_results."""
        try:
            config = QueryConfig(max_path_results=10)
            plan = connected_client.explain("RETURN 1 AS num", config)
            print(f"Explain with max_path_results=10: {plan if plan else '(empty)'}")
        except Exception as e:
            print(f"Explain with max_path_results error: {e}")

    def test_profile_with_max_path_results(self, connected_client: GqldbClient):
        """Test PROFILE with max_path_results."""
        try:
            config = QueryConfig(max_path_results=10)
            profile = connected_client.profile("RETURN 1 AS num", config)
            print(f"Profile with max_path_results=10: {profile if profile else '(empty)'}")
        except Exception as e:
            print(f"Profile with max_path_results error: {e}")



"""Health service handles health check operations."""

from __future__ import annotations

from typing import TYPE_CHECKING, Callable, Generator, Optional

import grpc

if TYPE_CHECKING:
    from ..types import HealthStatus
    from .service_context import ServiceContext


class HealthService:
    """Health service for checking service health."""

    def __init__(self, ctx: ServiceContext):
        self.ctx = ctx
        self._stub = None

    def _get_stub(self):
        if self._stub is None:
            from ..proto import gqldb_pb2_grpc
            channel = self.ctx.get_channel()
            self._stub = gqldb_pb2_grpc.HealthStub(channel)
        return self._stub

    def health_check(self, service: str) -> HealthStatus:
        from ..errors import GqldbError
        from ..proto import gqldb_pb2
        from .converters import convert_health_status

        try:
            request = gqldb_pb2.HealthCheckRequest(service=service)
            metadata = self.ctx.get_session_metadata()
            response = self._get_stub().Check(
                request, metadata=metadata, timeout=self.ctx.config.timeout
            )
            return convert_health_status(response.status)
        except grpc.RpcError as e:
            raise GqldbError(f"Health check failed: {e.details()}") from e

    def watch(
        self,
        service: str,
        callback: Optional[Callable[[HealthStatus], None]]
    ) -> None:
        from ..errors import GqldbError
        from ..proto import gqldb_pb2
        from .converters import convert_health_status

        try:
            request = gqldb_pb2.HealthCheckRequest(service=service)
            metadata = self.ctx.get_session_metadata()
            stream = self._get_stub().Watch(request, metadata=metadata)

            for response in stream:
                status = convert_health_status(response.status)
                if callback:
                    callback(status)
        except grpc.RpcError as e:
            raise GqldbError(f"Health watch failed: {e.details()}") from e

    def watch_iter(self, service: str) -> Generator[HealthStatus, None, None]:
        from ..errors import GqldbError
        from ..proto import gqldb_pb2
        from .converters import convert_health_status

        try:
            request = gqldb_pb2.HealthCheckRequest(service=service)
            metadata = self.ctx.get_session_metadata()
            stream = self._get_stub().Watch(request, metadata=metadata)

            for response in stream:
                yield convert_health_status(response.status)
        except grpc.RpcError as e:
            raise GqldbError(f"Health watch failed: {e.details()}") from e



"""Graph entity types for GQLDB Python driver."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Dict, List

@dataclass
class GqldbNode:
    """Graph node."""
    id: str = ""
    labels: List[str] = field(default_factory=list)
    properties: Dict[str, Any] = field(default_factory=dict)


@dataclass
class GqldbEdge:
    """Graph edge."""
    id: str = ""
    label: str = ""
    from_node_id: str = ""
    to_node_id: str = ""
    properties: Dict[str, Any] = field(default_factory=dict)


@dataclass
class GqldbPath:
    """Graph path consisting of nodes and edges."""
    nodes: List[GqldbNode] = field(default_factory=list)
    edges: List[GqldbEdge] = field(default_factory=list)


@dataclass
class GqldbError:
    """Error value from TRY/CATCH handling."""
    code: int = 0
    message: str = ""


@dataclass
class NodeData:
    """Data for a node to be inserted."""
    labels: List[str] = field(default_factory=list)
    properties: Dict[str, Any] = field(default_factory=dict)


@dataclass
class EdgeData:
    """Data for an edge to be inserted."""
    label: str = ""
    from_node_id: str = ""
    to_node_id: str = ""
    properties: Dict[str, Any] = field(default_factory=dict)


@dataclass
class GraphInfo:
    """Information about a graph."""
    name: str
    graph_type: GraphType
    node_count: int
    edge_count: int
    description: str


@dataclass
class TransactionInfo:
    """Information about a transaction."""
    transaction_id: int
    session_id: int
    graph_name: str
    read_only: bool
    created_at: int
    duration_ms: int
    internal_tx_id: str


@dataclass
class ASTCacheStats:
    """AST cache statistics."""
    hits: int
    misses: int
    evictions: int
    entries: int
    hit_rate: float


@dataclass
class PlanCacheStats:
    """Plan cache statistics."""
    size: int
    capacity: int
    hits: int
    misses: int
    hit_rate: float


@dataclass
class CacheStats:
    """Combined cache statistics."""
    ast_stats: Optional[ASTCacheStats] = None
    plan_stats: Optional[PlanCacheStats] = None


@dataclass
class Statistics:
    """Database statistics."""
    node_count: int
    edge_count: int
    label_counts: Dict[str, int] = field(default_factory=dict)
    edge_label_counts: Dict[str, int] = field(default_factory=dict)


@dataclass
class ExportedNode:
    """A node exported from the database."""
    id: str
    labels: List[str]
    properties: Dict[str, Any]


@dataclass
class ExportedEdge:
    """An edge exported from the database."""
    id: str
    label: str
    from_node_id: str
    to_node_id: str
    properties: Dict[str, Any]



"""Bulk import types for GQLDB Python driver."""

from dataclasses import dataclass, field
from typing import Optional

@dataclass
class BulkCreateNodesOptions:
    """Options for bulk node creation."""
    overwrite: bool = False


@dataclass
class BulkCreateEdgesOptions:
    """Options for bulk edge creation."""
    skip_invalid_nodes: bool = False


@dataclass
class CompactResult:
    """Result of a compact operation."""
    success: bool = False
    message: str = ""


@dataclass
class BulkImportSession:
    """Bulk import session result."""
    success: bool = False
    session_id: str = ""
    message: str = ""


@dataclass
class CheckpointResult:
    """Result of a checkpoint operation."""
    success: bool = False
    record_count: int = 0
    last_checkpoint_count: int = 0
    message: str = ""


@dataclass
class EndBulkImportResult:
    """Result of ending a bulk import session."""
    success: bool = False
    total_records: int = 0
    message: str = ""


@dataclass
class AbortBulkImportResult:
    """Result of aborting a bulk import session."""
    success: bool = False
    message: str = ""


@dataclass
class BulkImportStatus:
    """Status of a bulk import session."""
    is_active: bool = False
    graph_name: str = ""
    record_count: int = 0
    last_checkpoint_count: int = 0
    created_at: int = 0
    last_activity: int = 0



"""Transaction management for GQLDB Python driver."""

from __future__ import annotations

import threading
import time
from dataclasses import dataclass, field
from typing import Dict, List, Optional

from .errors import TransactionNotFoundError


@dataclass
class Transaction:
    """Represents an active database transaction."""
    id: int
    session_id: int
    graph_name: str
    read_only: bool = False
    created_at: float = field(default_factory=time.time)
    timeout: float = 0.0
    _committed: bool = field(default=False, repr=False)
    _rolled_back: bool = field(default=False, repr=False)
    _lock: threading.RLock = field(default_factory=threading.RLock, repr=False)

    @property
    def is_committed(self) -> bool:
        """Check if the transaction was committed."""
        with self._lock:
            return self._committed

    @property
    def is_rolled_back(self) -> bool:
        """Check if the transaction was rolled back."""
        with self._lock:
            return self._rolled_back

    @property
    def is_active(self) -> bool:
        """Check if the transaction is still active."""
        with self._lock:
            return not self._committed and not self._rolled_back

    def age(self) -> float:
        """Return how long the transaction has been active in seconds.

        Returns:
            Transaction age in seconds.
        """
        return time.time() - self.created_at

    def is_expired(self) -> bool:
        """Check if the transaction has exceeded its timeout.

        Returns:
            True if expired.
        """
        if self.timeout == 0:
            return False
        return self.age() > self.timeout


class TransactionManager:
    """Manages transactions for the client."""

    def __init__(self):
        self._transactions: Dict[int, Transaction] = {}
        self._lock = threading.RLock()

    def begin(
        self,
        tx_id: int,
        session_id: int,
        graph_name: str,
        read_only: bool,
        timeout: float
    ) -> Transaction:
        """Create a new transaction.

        Args:
            tx_id: Transaction ID from the server.
            session_id: Session ID.
            graph_name: Graph name for the transaction.
            read_only: Whether the transaction is read-only.
            timeout: Timeout in seconds.

        Returns:
            The created Transaction.
        """
        with self._lock:
            tx = Transaction(
                id=tx_id,
                session_id=session_id,
                graph_name=graph_name,
                read_only=read_only,
                timeout=timeout
            )
            self._transactions[tx_id] = tx
            return tx

    def commit(self, tx_id: int) -> None:
        """Mark a transaction as committed.

        Args:
            tx_id: Transaction ID.

        Raises:
            TransactionNotFoundError: If transaction not found.
        """
        with self._lock:
            tx = self._transactions.get(tx_id)
            if not tx:
                raise TransactionNotFoundError()

            with tx._lock:
                tx._committed = True

            del self._transactions[tx_id]

    def rollback(self, tx_id: int) -> None:
        """Mark a transaction as rolled back.

        Args:
            tx_id: Transaction ID.

        Raises:
            TransactionNotFoundError: If transaction not found.
        """
        with self._lock:
            tx = self._transactions.get(tx_id)
            if not tx:
                raise TransactionNotFoundError()

            with tx._lock:
                tx._rolled_back = True

            del self._transactions[tx_id]

    def get(self, tx_id: int) -> Optional[Transaction]:
        """Get a transaction by ID.

        Args:
            tx_id: Transaction ID.

        Returns:
            Transaction or None.
        """
        with self._lock:
            return self._transactions.get(tx_id)

    def get_active(self) -> List[Transaction]:
        """Get all active transactions.

        Returns:
            List of active transactions.
        """
        with self._lock:
            return list(self._transactions.values())

    def get_active_for_session(self, session_id: int) -> List[Transaction]:
        """Get all active transactions for a session.

        Args:
            session_id: Session ID.

        Returns:
            List of transactions for the session.
        """
        with self._lock:
            return [
                tx for tx in self._transactions.values()
                if tx.session_id == session_id
            ]

    def has_active(self) -> bool:
        """Check if there are any active transactions.

        Returns:
            True if there are active transactions.
        """
        with self._lock:
            return len(self._transactions) > 0

    def count(self) -> int:
        """Return the number of active transactions.

        Returns:
            Number of active transactions.
        """
        with self._lock:
            return len(self._transactions)

    def clear_all(self) -> None:
        """Clear all transactions (used during logout)."""
        with self._lock:
            self._transactions.clear()



"""Health Service integration tests."""

import pytest

from gqldb.client import GqldbClient
from gqldb.types import HealthStatus


@pytest.mark.integration
class TestHealth:
    """Tests for health check operations."""

    def test_health_check(self, connected_client: GqldbClient):
        """Test basic health check."""
        status = connected_client.health_check()
        assert status is not None
        assert status in [HealthStatus.SERVING, HealthStatus.UNKNOWN]
        print(f"Health check status: {status}")

    def test_health_check_with_service(self, connected_client: GqldbClient):
        """Test health check for specific service."""
        try:
            status = connected_client.health_check("gqldb")
            print(f"Health check for 'gqldb': {status}")
        except Exception as e:
            print(f"Health check with service name: {e}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gqldb.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'gqldb.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bgqldb.proto\x12\x05gqldb\"N\n\nTypedValue\x12!\n\x04type\x18\x01 \x01(\x0e\x32\x13.gqldb.PropertyType\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0f\n\x07is_null\x18\x03 \x01(\x08\";\n\tParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12 \n\x05value\x18\x02 \x01(\x0b\x32\x11.gqldb.TypedValue\"(\n\x03Row\x12!\n\x06values\x18\x01 \x03(\x0b\x32\x11.gqldb.TypedValue\"I\n\x0cLoginRequest\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x15\n\rdefault_graph\x18\x03 \x01(\t\"J\n\rLoginResponse\x12\x12\n\nsession_id\x18\x01 \x01(\x04\x12\x16\n\x0eserver_version\x18\x02 \x01(\t\x12\r\n\x05roles\x18\x03 \x03(\t\"\x0f\n\rLogoutRequest\"!\n\x0eLogoutResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\r\n\x0bPingRequest\"\"\n\x0cPingResponse\x12\x12\n\nlatency_ns\x18\x01 \x01(\x03\"\xbd\x01\n\nGqlRequest\x12\x0b\n\x03gql\x18\x01 \x01(\t\x12\x12\n\ngraph_name\x18\x02 \x01(\t\x12$\n\nparameters\x18\x03 \x03(\x0b\x32\x10.gqldb.Parameter\x12\x12\n\nsession_id\x18\x04 \x01(\x04\x12\x16\n\x0etransaction_id\x18\x05 \x01(\x04\x12\x0f\n\x07timeout\x18\x06 \x01(\x05\x12\x11\n\tread_only\x18\x07 \x01(\x08\x12\x18\n\x10max_path_results\x18\x08 \x01(\x03\"\x86\x01\n\x0bGqlResponse\x12\x0f\n\x07\x63olumns\x18\x01 \x03(\t\x12\x18\n\x04rows\x18\x02 \x03(\x0b\x32\n.gqldb.Row\x12\x11\n\trow_count\x18\x03 \x01(\x03\x12\x10\n\x08has_more\x18\x04 \x01(\x08\x12\x10\n\x08warnings\x18\x05 \x03(\t\x12\x15\n\rrows_affected\x18\x06 \x01(\x03\"\x1f\n\x0f\x45xplainResponse\x12\x0c\n\x04plan\x18\x01 \x01(\t\"\"\n\x0fProfileResponse\x12\x0f\n\x07profile\x18\x01 \x01(\t\"\xa1\x01\n\x08NodeData\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06labels\x18\x02 \x03(\t\x12\x33\n\nproperties\x18\x03 \x03(\x0b\x32\x1f.gqldb.NodeData.PropertiesEntry\x1a\x44\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12 \n\x05value\x18\x02 \x01(\x0b\x32\x11.gqldb.TypedValue:\x02\x38\x01\"\xbe\x01\n\x08\x45\x64geData\x12\r\n\x05label\x18\x01 \x01(\t\x12\x14\n\x0c\x66rom_node_id\x18\x02 \x01(\t\x12\x12\n\nto_node_id\x18\x03 \x01(\t\x12\x33\n\nproperties\x18\x04 \x03(\x0b\x32\x1f.gqldb.EdgeData.PropertiesEntry\x1a\x44\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12 \n\x05value\x18\x02 \x01(\x0b\x32\x11.gqldb.TypedValue:\x02\x38\x01\"+\n\x16\x42ulkCreateNodesOptions\x12\x11\n\toverwrite\x18\x01 \x01(\x08\"4\n\x16\x42ulkCreateEdgesOptions\x12\x1a\n\x12skip_invalid_nodes\x18\x01 \x01(\x08\"\x98\x01\n\x12InsertNodesRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x1e\n\x05nodes\x18\x02 \x03(\x0b\x32\x0f.gqldb.NodeData\x12.\n\x07options\x18\x03 \x01(\x0b\x32\x1d.gqldb.BulkCreateNodesOptions\x12\x1e\n\x16\x62ulk_import_session_id\x18\x04 \x01(\t\"]\n\x13InsertNodesResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08node_ids\x18\x02 \x03(\t\x12\x12\n\nnode_count\x18\x03 \x01(\x03\x12\x0f\n\x07message\x18\x04 \x01(\t\"\x98\x01\n\x12InsertEdgesRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x1e\n\x05\x65\x64ges\x18\x02 \x03(\x0b\x32\x0f.gqldb.EdgeData\x12.\n\x07options\x18\x03 \x01(\x0b\x32\x1d.gqldb.BulkCreateEdgesOptions\x12\x1e\n\x16\x62ulk_import_session_id\x18\x04 \x01(\t\"t\n\x13InsertEdgesResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08\x65\x64ge_ids\x18\x02 \x03(\t\x12\x12\n\nedge_count\x18\x03 \x01(\x03\x12\x0f\n\x07message\x18\x04 \x01(\t\x12\x15\n\rskipped_count\x18\x05 \x01(\x03\"Y\n\x12\x44\x65leteNodesRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x10\n\x08node_ids\x18\x02 \x03(\t\x12\x0e\n\x06labels\x18\x03 \x03(\t\x12\r\n\x05where\x18\x04 \x01(\t\"N\n\x13\x44\x65leteNodesResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rdeleted_count\x18\x02 \x01(\x03\x12\x0f\n\x07message\x18\x03 \x01(\t\"X\n\x12\x44\x65leteEdgesRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x10\n\x08\x65\x64ge_ids\x18\x02 \x03(\t\x12\r\n\x05label\x18\x03 \x01(\t\x12\r\n\x05where\x18\x04 \x01(\t\"N\n\x13\x44\x65leteEdgesResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rdeleted_count\x18\x02 \x01(\x03\x12\x0f\n\x07message\x18\x03 \x01(\t\"\xa7\x01\n\rExportRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x12\n\nbatch_size\x18\x02 \x01(\x05\x12\x14\n\x0c\x65xport_nodes\x18\x03 \x01(\x08\x12\x14\n\x0c\x65xport_edges\x18\x04 \x01(\x08\x12\x13\n\x0bnode_labels\x18\x05 \x03(\t\x12\x13\n\x0b\x65\x64ge_labels\x18\x06 \x03(\t\x12\x18\n\x10include_metadata\x18\x07 \x01(\x08\"S\n\x0e\x45xportResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x10\n\x08is_final\x18\x02 \x01(\x08\x12!\n\x05stats\x18\x03 \x01(\x0b\x32\x12.gqldb.ExportStats\"i\n\x0b\x45xportStats\x12\x16\n\x0enodes_exported\x18\x01 \x01(\x03\x12\x16\n\x0e\x65\x64ges_exported\x18\x02 \x01(\x03\x12\x15\n\rbytes_written\x18\x03 \x01(\x03\x12\x13\n\x0b\x64uration_ms\x18\x04 \x01(\x03\"]\n\x12\x43reateGraphRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\ngraph_type\x18\x02 \x01(\x0e\x32\x10.gqldb.GraphType\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\"7\n\x13\x43reateGraphResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"3\n\x10\x44ropGraphRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tif_exists\x18\x02 \x01(\x08\"5\n\x11\x44ropGraphResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"7\n\x0fUseGraphRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\nsession_id\x18\x02 \x01(\x04\x42\x02\x18\x01\"4\n\x10UseGraphResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x13\n\x11ListGraphsRequest\"6\n\x12ListGraphsResponse\x12 \n\x06graphs\x18\x01 \x03(\x0b\x32\x10.gqldb.GraphInfo\"#\n\x13GetGraphInfoRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"6\n\x14GetGraphInfoResponse\x12\x1e\n\x04info\x18\x01 \x01(\x0b\x32\x10.gqldb.GraphInfo\"|\n\tGraphInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\ngraph_type\x18\x02 \x01(\x0e\x32\x10.gqldb.GraphType\x12\x12\n\nnode_count\x18\x03 \x01(\x03\x12\x12\n\nedge_count\x18\x04 \x01(\x03\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\"^\n\x0c\x42\x65ginRequest\x12\x16\n\nsession_id\x18\x01 \x01(\x04\x42\x02\x18\x01\x12\x12\n\ngraph_name\x18\x02 \x01(\t\x12\x11\n\tread_only\x18\x03 \x01(\x08\x12\x0f\n\x07timeout\x18\x04 \x01(\x05\"8\n\rBeginResponse\x12\x16\n\x0etransaction_id\x18\x01 \x01(\x04\x12\x0f\n\x07message\x18\x02 \x01(\t\"?\n\rCommitRequest\x12\x16\n\nsession_id\x18\x01 \x01(\x04\x42\x02\x18\x01\x12\x16\n\x0etransaction_id\x18\x02 \x01(\x04\"2\n\x0e\x43ommitResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"A\n\x0fRollbackRequest\x12\x16\n\nsession_id\x18\x01 \x01(\x04\x42\x02\x18\x01\x12\x16\n\x0etransaction_id\x18\x02 \x01(\x04\"4\n\x10RollbackResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"K\n\x17ListTransactionsRequest\x12\x16\n\nsession_id\x18\x01 \x01(\x04\x42\x02\x18\x01\x12\x18\n\x10\x61ll_transactions\x18\x02 \x01(\x08\"H\n\x18ListTransactionsResponse\x12,\n\x0ctransactions\x18\x01 \x03(\x0b\x32\x16.gqldb.TransactionInfo\"\xa5\x01\n\x0fTransactionInfo\x12\x16\n\x0etransaction_id\x18\x01 \x01(\x04\x12\x12\n\nsession_id\x18\x02 \x01(\x04\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12\x11\n\tread_only\x18\x04 \x01(\x08\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x13\n\x0b\x64uration_ms\x18\x06 \x01(\x03\x12\x16\n\x0einternal_tx_id\x18\x07 \x01(\t\"%\n\x12HealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\":\n\x13HealthCheckResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.gqldb.HealthStatus\"$\n\x13WarmupParserRequest\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\"8\n\x14WarmupParserResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"<\n\x14GetCacheStatsRequest\x12$\n\ncache_type\x18\x01 \x01(\x0e\x32\x10.gqldb.CacheType\"h\n\x12\x43\x61\x63heStatsResponse\x12\'\n\tast_stats\x18\x01 \x01(\x0b\x32\x14.gqldb.ASTCacheStats\x12)\n\nplan_stats\x18\x02 \x01(\x0b\x32\x15.gqldb.PlanCacheStats\"c\n\rASTCacheStats\x12\x0c\n\x04hits\x18\x01 \x01(\x04\x12\x0e\n\x06misses\x18\x02 \x01(\x04\x12\x11\n\tevictions\x18\x03 \x01(\x04\x12\x0f\n\x07\x65ntries\x18\x04 \x01(\x05\x12\x10\n\x08hit_rate\x18\x05 \x01(\x01\"`\n\x0ePlanCacheStats\x12\x0c\n\x04size\x18\x01 \x01(\x05\x12\x10\n\x08\x63\x61pacity\x18\x02 \x01(\x05\x12\x0c\n\x04hits\x18\x03 \x01(\x04\x12\x0e\n\x06misses\x18\x04 \x01(\x04\x12\x10\n\x08hit_rate\x18\x05 \x01(\x01\"9\n\x11\x43learCacheRequest\x12$\n\ncache_type\x18\x01 \x01(\x0e\x32\x10.gqldb.CacheType\"6\n\x12\x43learCacheResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"*\n\x14GetStatisticsRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\"\xbe\x02\n\x15GetStatisticsResponse\x12\x12\n\nnode_count\x18\x01 \x01(\x04\x12\x12\n\nedge_count\x18\x02 \x01(\x04\x12\x43\n\x0clabel_counts\x18\x03 \x03(\x0b\x32-.gqldb.GetStatisticsResponse.LabelCountsEntry\x12L\n\x11\x65\x64ge_label_counts\x18\x04 \x03(\x0b\x32\x31.gqldb.GetStatisticsResponse.EdgeLabelCountsEntry\x1a\x32\n\x10LabelCountsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x04:\x02\x38\x01\x1a\x36\n\x14\x45\x64geLabelCountsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x04:\x02\x38\x01\"4\n InvalidatePermissionCacheRequest\x12\x10\n\x08username\x18\x01 \x01(\t\"4\n!InvalidatePermissionCacheResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\x10\n\x0e\x43ompactRequest\"3\n\x0f\x43ompactResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"G\n\x1dWaitForComputeTopologyRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x12\n\ntimeout_ms\x18\x02 \x01(\x03\"@\n\x1eWaitForComputeTopologyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xa6\x01\n\x16StartBulkImportRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\x12\x18\n\x10\x63heckpoint_every\x18\x02 \x01(\x03\x12\x17\n\x0f\x65stimated_nodes\x18\x03 \x01(\x03\x12\x17\n\x0f\x65stimated_edges\x18\x04 \x01(\x03\x12\x15\n\rmemtable_size\x18\x05 \x01(\x03\x12\x15\n\rmax_memtables\x18\x06 \x01(\x05\"O\n\x17StartBulkImportResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\"\'\n\x11\x43heckpointRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"k\n\x12\x43heckpointResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x14\n\x0crecord_count\x18\x02 \x01(\x03\x12\x1d\n\x15last_checkpoint_count\x18\x03 \x01(\x03\x12\x0f\n\x07message\x18\x04 \x01(\t\"*\n\x14\x45ndBulkImportRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"P\n\x15\x45ndBulkImportResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rtotal_records\x18\x02 \x01(\x03\x12\x0f\n\x07message\x18\x03 \x01(\t\",\n\x16\x41\x62ortBulkImportRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\";\n\x17\x41\x62ortBulkImportResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"0\n\x1aGetBulkImportStatusRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"\xa4\x01\n\x1bGetBulkImportStatusResponse\x12\x11\n\tis_active\x18\x01 \x01(\x08\x12\x12\n\ngraph_name\x18\x02 \x01(\t\x12\x14\n\x0crecord_count\x18\x03 \x01(\x03\x12\x1d\n\x15last_checkpoint_count\x18\x04 \x01(\x03\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x15\n\rlast_activity\x18\x06 \x01(\x03*\x90\x07\n\x0cPropertyType\x12\x17\n\x13PROPERTY_TYPE_UNSET\x10\x00\x12\x17\n\x13PROPERTY_TYPE_INT32\x10\x01\x12\x18\n\x14PROPERTY_TYPE_UINT32\x10\x02\x12\x17\n\x13PROPERTY_TYPE_INT64\x10\x03\x12\x18\n\x14PROPERTY_TYPE_UINT64\x10\x04\x12\x17\n\x13PROPERTY_TYPE_FLOAT\x10\x05\x12\x18\n\x14PROPERTY_TYPE_DOUBLE\x10\x06\x12\x18\n\x14PROPERTY_TYPE_STRING\x10\x07\x12\x1a\n\x16PROPERTY_TYPE_DATETIME\x10\x08\x12\x1b\n\x17PROPERTY_TYPE_TIMESTAMP\x10\t\x12\x16\n\x12PROPERTY_TYPE_TEXT\x10\n\x12\x16\n\x12PROPERTY_TYPE_BLOB\x10\x0b\x12\x17\n\x13PROPERTY_TYPE_POINT\x10\x0c\x12\x19\n\x15PROPERTY_TYPE_DECIMAL\x10\r\x12\x16\n\x12PROPERTY_TYPE_LIST\x10\x0e\x12\x15\n\x11PROPERTY_TYPE_SET\x10\x0f\x12\x15\n\x11PROPERTY_TYPE_MAP\x10\x10\x12\x16\n\x12PROPERTY_TYPE_NULL\x10\x11\x12\x16\n\x12PROPERTY_TYPE_BOOL\x10\x12\x12 \n\x1cPROPERTY_TYPE_LOCAL_DATETIME\x10\x13\x12 \n\x1cPROPERTY_TYPE_ZONED_DATETIME\x10\x14\x12\x16\n\x12PROPERTY_TYPE_DATE\x10\x15\x12\x1c\n\x18PROPERTY_TYPE_ZONED_TIME\x10\x16\x12\x1c\n\x18PROPERTY_TYPE_LOCAL_TIME\x10\x17\x12\x1f\n\x1bPROPERTY_TYPE_YEAR_TO_MONTH\x10\x18\x12\x1f\n\x1bPROPERTY_TYPE_DAY_TO_SECOND\x10\x19\x12\x18\n\x14PROPERTY_TYPE_RECORD\x10\x1a\x12\x19\n\x15PROPERTY_TYPE_POINT3D\x10\x1b\x12\x18\n\x14PROPERTY_TYPE_VECTOR\x10\x1c\x12\x17\n\x13PROPERTY_TYPE_TABLE\x10\x1d\x12\x16\n\x12PROPERTY_TYPE_PATH\x10\x1e\x12\x17\n\x13PROPERTY_TYPE_ERROR\x10\x1f\x12\x16\n\x12PROPERTY_TYPE_NODE\x10 \x12\x16\n\x12PROPERTY_TYPE_EDGE\x10!*P\n\tGraphType\x12\x13\n\x0fGRAPH_TYPE_OPEN\x10\x00\x12\x15\n\x11GRAPH_TYPE_CLOSED\x10\x01\x12\x17\n\x13GRAPH_TYPE_ONTOLOGY\x10\x02*\x86\x01\n\x0cHealthStatus\x12\x19\n\x15HEALTH_STATUS_UNKNOWN\x10\x00\x12\x19\n\x15HEALTH_STATUS_SERVING\x10\x01\x12\x1d\n\x19HEALTH_STATUS_NOT_SERVING\x10\x02\x12!\n\x1dHEALTH_STATUS_SERVICE_UNKNOWN\x10\x03*H\n\tCacheType\x12\x12\n\x0e\x43\x41\x43HE_TYPE_ALL\x10\x00\x12\x12\n\x0e\x43\x41\x43HE_TYPE_AST\x10\x01\x12\x13\n\x0f\x43\x41\x43HE_TYPE_PLAN\x10\x02\x32\xac\x01\n\x0eSessionService\x12\x32\n\x05Login\x12\x13.gqldb.LoginRequest\x1a\x14.gqldb.LoginResponse\x12\x35\n\x06Logout\x12\x14.gqldb.LogoutRequest\x1a\x15.gqldb.LogoutResponse\x12/\n\x04Ping\x12\x12.gqldb.PingRequest\x1a\x13.gqldb.PingResponse2\xde\x01\n\x0cQueryService\x12,\n\x03Gql\x12\x11.gqldb.GqlRequest\x1a\x12.gqldb.GqlResponse\x12\x34\n\tGqlStream\x12\x11.gqldb.GqlRequest\x1a\x12.gqldb.GqlResponse0\x01\x12\x34\n\x07\x45xplain\x12\x11.gqldb.GqlRequest\x1a\x16.gqldb.ExplainResponse\x12\x34\n\x07Profile\x12\x11.gqldb.GqlRequest\x1a\x16.gqldb.ProfileResponse2\xde\x02\n\x0b\x44\x61taService\x12\x44\n\x0bInsertNodes\x12\x19.gqldb.InsertNodesRequest\x1a\x1a.gqldb.InsertNodesResponse\x12\x44\n\x0bInsertEdges\x12\x19.gqldb.InsertEdgesRequest\x1a\x1a.gqldb.InsertEdgesResponse\x12\x44\n\x0b\x44\x65leteNodes\x12\x19.gqldb.DeleteNodesRequest\x1a\x1a.gqldb.DeleteNodesResponse\x12\x44\n\x0b\x44\x65leteEdges\x12\x19.gqldb.DeleteEdgesRequest\x1a\x1a.gqldb.DeleteEdgesResponse\x12\x37\n\x06\x45xport\x12\x14.gqldb.ExportRequest\x1a\x15.gqldb.ExportResponse0\x01\x32\xdd\x02\n\x0cGraphService\x12\x44\n\x0b\x43reateGraph\x12\x19.gqldb.CreateGraphRequest\x1a\x1a.gqldb.CreateGraphResponse\x12>\n\tDropGraph\x12\x17.gqldb.DropGraphRequest\x1a\x18.gqldb.DropGraphResponse\x12;\n\x08UseGraph\x12\x16.gqldb.UseGraphRequest\x1a\x17.gqldb.UseGraphResponse\x12\x41\n\nListGraphs\x12\x18.gqldb.ListGraphsRequest\x1a\x19.gqldb.ListGraphsResponse\x12G\n\x0cGetGraphInfo\x12\x1a.gqldb.GetGraphInfoRequest\x1a\x1b.gqldb.GetGraphInfoResponse2\x91\x02\n\x12TransactionService\x12\x32\n\x05\x42\x65gin\x12\x13.gqldb.BeginRequest\x1a\x14.gqldb.BeginResponse\x12\x35\n\x06\x43ommit\x12\x14.gqldb.CommitRequest\x1a\x15.gqldb.CommitResponse\x12;\n\x08Rollback\x12\x16.gqldb.RollbackRequest\x1a\x17.gqldb.RollbackResponse\x12S\n\x10ListTransactions\x12\x1e.gqldb.ListTransactionsRequest\x1a\x1f.gqldb.ListTransactionsResponse2\x8a\x01\n\x06Health\x12>\n\x05\x43heck\x12\x19.gqldb.HealthCheckRequest\x1a\x1a.gqldb.HealthCheckResponse\x12@\n\x05Watch\x12\x19.gqldb.HealthCheckRequest\x1a\x1a.gqldb.HealthCheckResponse0\x01\x32\xc0\x04\n\x0c\x41\x64minService\x12G\n\x0cWarmupParser\x12\x1a.gqldb.WarmupParserRequest\x1a\x1b.gqldb.WarmupParserResponse\x12G\n\rGetCacheStats\x12\x1b.gqldb.GetCacheStatsRequest\x1a\x19.gqldb.CacheStatsResponse\x12\x41\n\nClearCache\x12\x18.gqldb.ClearCacheRequest\x1a\x19.gqldb.ClearCacheResponse\x12J\n\rGetStatistics\x12\x1b.gqldb.GetStatisticsRequest\x1a\x1c.gqldb.GetStatisticsResponse\x12n\n\x19InvalidatePermissionCache\x12\'.gqldb.InvalidatePermissionCacheRequest\x1a(.gqldb.InvalidatePermissionCacheResponse\x12\x38\n\x07\x43ompact\x12\x15.gqldb.CompactRequest\x1a\x16.gqldb.CompactResponse\x12\x65\n\x16WaitForComputeTopology\x12$.gqldb.WaitForComputeTopologyRequest\x1a%.gqldb.WaitForComputeTopologyResponse2\xa4\x03\n\x11\x42ulkImportService\x12P\n\x0fStartBulkImport\x12\x1d.gqldb.StartBulkImportRequest\x1a\x1e.gqldb.StartBulkImportResponse\x12\x41\n\nCheckpoint\x12\x18.gqldb.CheckpointRequest\x1a\x19.gqldb.CheckpointResponse\x12J\n\rEndBulkImport\x12\x1b.gqldb.EndBulkImportRequest\x1a\x1c.gqldb.EndBulkImportResponse\x12P\n\x0f\x41\x62ortBulkImport\x12\x1d.gqldb.AbortBulkImportRequest\x1a\x1e.gqldb.AbortBulkImportResponse\x12\\\n\x13GetBulkImportStatus\x12!.gqldb.GetBulkImportStatusRequest\x1a\".gqldb.GetBulkImportStatusResponseB\'Z%github.com/zhangjsff/gqldb-grpc/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gqldb_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z%github.com/zhangjsff/gqldb-grpc/proto'
  _globals['_NODEDATA_PROPERTIESENTRY']._loaded_options = None
  _globals['_NODEDATA_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_EDGEDATA_PROPERTIESENTRY']._loaded_options = None
  _globals['_EDGEDATA_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_USEGRAPHREQUEST'].fields_by_name['session_id']._loaded_options = None
  _globals['_USEGRAPHREQUEST'].fields_by_name['session_id']._serialized_options = b'\030\001'
  _globals['_BEGINREQUEST'].fields_by_name['session_id']._loaded_options = None
  _globals['_BEGINREQUEST'].fields_by_name['session_id']._serialized_options = b'\030\001'
  _globals['_COMMITREQUEST'].fields_by_name['session_id']._loaded_options = None
  _globals['_COMMITREQUEST'].fields_by_name['session_id']._serialized_options = b'\030\001'
  _globals['_ROLLBACKREQUEST'].fields_by_name['session_id']._loaded_options = None
  _globals['_ROLLBACKREQUEST'].fields_by_name['session_id']._serialized_options = b'\030\001'
  _globals['_LISTTRANSACTIONSREQUEST'].fields_by_name['session_id']._loaded_options = None
  _globals['_LISTTRANSACTIONSREQUEST'].fields_by_name['session_id']._serialized_options = b'\030\001'
  _globals['_GETSTATISTICSRESPONSE_LABELCOUNTSENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_LABELCOUNTSENTRY']._serialized_options = b'8\001'
  _globals['_GETSTATISTICSRESPONSE_EDGELABELCOUNTSENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_EDGELABELCOUNTSENTRY']._serialized_options = b'8\001'
  _globals['_PROPERTYTYPE']._serialized_start=6128
  _globals['_PROPERTYTYPE']._serialized_end=7040
  _globals['_GRAPHTYPE']._serialized_start=7042
  _globals['_GRAPHTYPE']._serialized_end=7122
  _globals['_HEALTHSTATUS']._serialized_start=7125
  _globals['_HEALTHSTATUS']._serialized_end=7259
  _globals['_CACHETYPE']._serialized_start=7261
  _globals['_CACHETYPE']._serialized_end=7333
  _globals['_TYPEDVALUE']._serialized_start=22
  _globals['_TYPEDVALUE']._serialized_end=100
  _globals['_PARAMETER']._serialized_start=102
  _globals['_PARAMETER']._serialized_end=161
  _globals['_ROW']._serialized_start=163
  _globals['_ROW']._serialized_end=203
  _globals['_LOGINREQUEST']._serialized_start=205
  _globals['_LOGINREQUEST']._serialized_end=278
  _globals['_LOGINRESPONSE']._serialized_start=280
  _globals['_LOGINRESPONSE']._serialized_end=354
  _globals['_LOGOUTREQUEST']._serialized_start=356
  _globals['_LOGOUTREQUEST']._serialized_end=371
  _globals['_LOGOUTRESPONSE']._serialized_start=373
  _globals['_LOGOUTRESPONSE']._serialized_end=406
  _globals['_PINGREQUEST']._serialized_start=408
  _globals['_PINGREQUEST']._serialized_end=421
  _globals['_PINGRESPONSE']._serialized_start=423
  _globals['_PINGRESPONSE']._serialized_end=457
  _globals['_GQLREQUEST']._serialized_start=460
  _globals['_GQLREQUEST']._serialized_end=649
  _globals['_GQLRESPONSE']._serialized_start=652
  _globals['_GQLRESPONSE']._serialized_end=786
  _globals['_EXPLAINRESPONSE']._serialized_start=788
  _globals['_EXPLAINRESPONSE']._serialized_end=819
  _globals['_PROFILERESPONSE']._serialized_start=821
  _globals['_PROFILERESPONSE']._serialized_end=855
  _globals['_NODEDATA']._serialized_start=858
  _globals['_NODEDATA']._serialized_end=1019
  _globals['_NODEDATA_PROPERTIESENTRY']._serialized_start=951
  _globals['_NODEDATA_PROPERTIESENTRY']._serialized_end=1019
  _globals['_EDGEDATA']._serialized_start=1022
  _globals['_EDGEDATA']._serialized_end=1212
  _globals['_EDGEDATA_PROPERTIESENTRY']._serialized_start=951
  _globals['_EDGEDATA_PROPERTIESENTRY']._serialized_end=1019
  _globals['_BULKCREATENODESOPTIONS']._serialized_start=1214
  _globals['_BULKCREATENODESOPTIONS']._serialized_end=1257
  _globals['_BULKCREATEEDGESOPTIONS']._serialized_start=1259
  _globals['_BULKCREATEEDGESOPTIONS']._serialized_end=1311
  _globals['_INSERTNODESREQUEST']._serialized_start=1314
  _globals['_INSERTNODESREQUEST']._serialized_end=1466
  _globals['_INSERTNODESRESPONSE']._serialized_start=1468
  _globals['_INSERTNODESRESPONSE']._serialized_end=1561
  _globals['_INSERTEDGESREQUEST']._serialized_start=1564
  _globals['_INSERTEDGESREQUEST']._serialized_end=1716
  _globals['_INSERTEDGESRESPONSE']._serialized_start=1718
  _globals['_INSERTEDGESRESPONSE']._serialized_end=1834
  _globals['_DELETENODESREQUEST']._serialized_start=1836
  _globals['_DELETENODESREQUEST']._serialized_end=1925
  _globals['_DELETENODESRESPONSE']._serialized_start=1927
  _globals['_DELETENODESRESPONSE']._serialized_end=2005
  _globals['_DELETEEDGESREQUEST']._serialized_start=2007
  _globals['_DELETEEDGESREQUEST']._serialized_end=2095
  _globals['_DELETEEDGESRESPONSE']._serialized_start=2097
  _globals['_DELETEEDGESRESPONSE']._serialized_end=2175
  _globals['_EXPORTREQUEST']._serialized_start=2178
  _globals['_EXPORTREQUEST']._serialized_end=2345
  _globals['_EXPORTRESPONSE']._serialized_start=2347
  _globals['_EXPORTRESPONSE']._serialized_end=2430
  _globals['_EXPORTSTATS']._serialized_start=2432
  _globals['_EXPORTSTATS']._serialized_end=2537
  _globals['_CREATEGRAPHREQUEST']._serialized_start=2539
  _globals['_CREATEGRAPHREQUEST']._serialized_end=2632
  _globals['_CREATEGRAPHRESPONSE']._serialized_start=2634
  _globals['_CREATEGRAPHRESPONSE']._serialized_end=2689
  _globals['_DROPGRAPHREQUEST']._serialized_start=2691
  _globals['_DROPGRAPHREQUEST']._serialized_end=2742
  _globals['_DROPGRAPHRESPONSE']._serialized_start=2744
  _globals['_DROPGRAPHRESPONSE']._serialized_end=2797
  _globals['_USEGRAPHREQUEST']._serialized_start=2799
  _globals['_USEGRAPHREQUEST']._serialized_end=2854
  _globals['_USEGRAPHRESPONSE']._serialized_start=2856
  _globals['_USEGRAPHRESPONSE']._serialized_end=2908
  _globals['_LISTGRAPHSREQUEST']._serialized_start=2910
  _globals['_LISTGRAPHSREQUEST']._serialized_end=2929
  _globals['_LISTGRAPHSRESPONSE']._serialized_start=2931
  _globals['_LISTGRAPHSRESPONSE']._serialized_end=2985
  _globals['_GETGRAPHINFOREQUEST']._serialized_start=2987
  _globals['_GETGRAPHINFOREQUEST']._serialized_end=3022
  _globals['_GETGRAPHINFORESPONSE']._serialized_start=3024
  _globals['_GETGRAPHINFORESPONSE']._serialized_end=3078
  _globals['_GRAPHINFO']._serialized_start=3080
  _globals['_GRAPHINFO']._serialized_end=3204
  _globals['_BEGINREQUEST']._serialized_start=3206
  _globals['_BEGINREQUEST']._serialized_end=3300
  _globals['_BEGINRESPONSE']._serialized_start=3302
  _globals['_BEGINRESPONSE']._serialized_end=3358
  _globals['_COMMITREQUEST']._serialized_start=3360
  _globals['_COMMITREQUEST']._serialized_end=3423
  _globals['_COMMITRESPONSE']._serialized_start=3425
  _globals['_COMMITRESPONSE']._serialized_end=3475
  _globals['_ROLLBACKREQUEST']._serialized_start=3477
  _globals['_ROLLBACKREQUEST']._serialized_end=3542
  _globals['_ROLLBACKRESPONSE']._serialized_start=3544
  _globals['_ROLLBACKRESPONSE']._serialized_end=3596
  _globals['_LISTTRANSACTIONSREQUEST']._serialized_start=3598
  _globals['_LISTTRANSACTIONSREQUEST']._serialized_end=3673
  _globals['_LISTTRANSACTIONSRESPONSE']._serialized_start=3675
  _globals['_LISTTRANSACTIONSRESPONSE']._serialized_end=3747
  _globals['_TRANSACTIONINFO']._serialized_start=3750
  _globals['_TRANSACTIONINFO']._serialized_end=3915
  _globals['_HEALTHCHECKREQUEST']._serialized_start=3917
  _globals['_HEALTHCHECKREQUEST']._serialized_end=3954
  _globals['_HEALTHCHECKRESPONSE']._serialized_start=3956
  _globals['_HEALTHCHECKRESPONSE']._serialized_end=4014
  _globals['_WARMUPPARSERREQUEST']._serialized_start=4016
  _globals['_WARMUPPARSERREQUEST']._serialized_end=4052
  _globals['_WARMUPPARSERRESPONSE']._serialized_start=4054
  _globals['_WARMUPPARSERRESPONSE']._serialized_end=4110
  _globals['_GETCACHESTATSREQUEST']._serialized_start=4112
  _globals['_GETCACHESTATSREQUEST']._serialized_end=4172
  _globals['_CACHESTATSRESPONSE']._serialized_start=4174
  _globals['_CACHESTATSRESPONSE']._serialized_end=4278
  _globals['_ASTCACHESTATS']._serialized_start=4280
  _globals['_ASTCACHESTATS']._serialized_end=4379
  _globals['_PLANCACHESTATS']._serialized_start=4381
  _globals['_PLANCACHESTATS']._serialized_end=4477
  _globals['_CLEARCACHEREQUEST']._serialized_start=4479
  _globals['_CLEARCACHEREQUEST']._serialized_end=4536
  _globals['_CLEARCACHERESPONSE']._serialized_start=4538
  _globals['_CLEARCACHERESPONSE']._serialized_end=4592
  _globals['_GETSTATISTICSREQUEST']._serialized_start=4594
  _globals['_GETSTATISTICSREQUEST']._serialized_end=4636
  _globals['_GETSTATISTICSRESPONSE']._serialized_start=4639
  _globals['_GETSTATISTICSRESPONSE']._serialized_end=4957
  _globals['_GETSTATISTICSRESPONSE_LABELCOUNTSENTRY']._serialized_start=4851
  _globals['_GETSTATISTICSRESPONSE_LABELCOUNTSENTRY']._serialized_end=4901
  _globals['_GETSTATISTICSRESPONSE_EDGELABELCOUNTSENTRY']._serialized_start=4903
  _globals['_GETSTATISTICSRESPONSE_EDGELABELCOUNTSENTRY']._serialized_end=4957
  _globals['_INVALIDATEPERMISSIONCACHEREQUEST']._serialized_start=4959
  _globals['_INVALIDATEPERMISSIONCACHEREQUEST']._serialized_end=5011
  _globals['_INVALIDATEPERMISSIONCACHERESPONSE']._serialized_start=5013
  _globals['_INVALIDATEPERMISSIONCACHERESPONSE']._serialized_end=5065
  _globals['_COMPACTREQUEST']._serialized_start=5067
  _globals['_COMPACTREQUEST']._serialized_end=5083
  _globals['_COMPACTRESPONSE']._serialized_start=5085
  _globals['_COMPACTRESPONSE']._serialized_end=5136
  _globals['_WAITFORCOMPUTETOPOLOGYREQUEST']._serialized_start=5138
  _globals['_WAITFORCOMPUTETOPOLOGYREQUEST']._serialized_end=5209
  _globals['_WAITFORCOMPUTETOPOLOGYRESPONSE']._serialized_start=5211
  _globals['_WAITFORCOMPUTETOPOLOGYRESPONSE']._serialized_end=5275
  _globals['_STARTBULKIMPORTREQUEST']._serialized_start=5278
  _globals['_STARTBULKIMPORTREQUEST']._serialized_end=5444
  _globals['_STARTBULKIMPORTRESPONSE']._serialized_start=5446
  _globals['_STARTBULKIMPORTRESPONSE']._serialized_end=5525
  _globals['_CHECKPOINTREQUEST']._serialized_start=5527
  _globals['_CHECKPOINTREQUEST']._serialized_end=5566
  _globals['_CHECKPOINTRESPONSE']._serialized_start=5568
  _globals['_CHECKPOINTRESPONSE']._serialized_end=5675
  _globals['_ENDBULKIMPORTREQUEST']._serialized_start=5677
  _globals['_ENDBULKIMPORTREQUEST']._serialized_end=5719
  _globals['_ENDBULKIMPORTRESPONSE']._serialized_start=5721
  _globals['_ENDBULKIMPORTRESPONSE']._serialized_end=5801
  _globals['_ABORTBULKIMPORTREQUEST']._serialized_start=5803
  _globals['_ABORTBULKIMPORTREQUEST']._serialized_end=5847
  _globals['_ABORTBULKIMPORTRESPONSE']._serialized_start=5849
  _globals['_ABORTBULKIMPORTRESPONSE']._serialized_end=5908
  _globals['_GETBULKIMPORTSTATUSREQUEST']._serialized_start=5910
  _globals['_GETBULKIMPORTSTATUSREQUEST']._serialized_end=5958
  _globals['_GETBULKIMPORTSTATUSRESPONSE']._serialized_start=5961
  _globals['_GETBULKIMPORTSTATUSRESPONSE']._serialized_end=6125
  _globals['_SESSIONSERVICE']._serialized_start=7336
  _globals['_SESSIONSERVICE']._serialized_end=7508
  _globals['_QUERYSERVICE']._serialized_start=7511
  _globals['_QUERYSERVICE']._serialized_end=7733
  _globals['_DATASERVICE']._serialized_start=7736
  _globals['_DATASERVICE']._serialized_end=8086
  _globals['_GRAPHSERVICE']._serialized_start=8089
  _globals['_GRAPHSERVICE']._serialized_end=8438
  _globals['_TRANSACTIONSERVICE']._serialized_start=8441
  _globals['_TRANSACTIONSERVICE']._serialized_end=8714
  _globals['_HEALTH']._serialized_start=8717
  _globals['_HEALTH']._serialized_end=8855
  _globals['_ADMINSERVICE']._serialized_start=8858
  _globals['_ADMINSERVICE']._serialized_end=9434
  _globals['_BULKIMPORTSERVICE']._serialized_start=9437
  _globals['_BULKIMPORTSERVICE']._serialized_end=9857
# @@protoc_insertion_point(module_scope)



"""Schema types for GQLDB Python driver."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, List

from .enums import PropertyType

@dataclass
class PropertyDef:
    """Property definition in a schema."""
    name: str
    type: PropertyType


@dataclass
class Schema:
    """Node/edge schema metadata."""
    name: str
    properties: List[PropertyDef] = field(default_factory=list)


@dataclass
class Header:
    """Column header in a table."""
    name: str
    type: PropertyType = PropertyType.UNSET


@dataclass
class Table:
    """Generic result table."""
    name: str = ""
    headers: List[Header] = field(default_factory=list)
    rows: List[List[Any]] = field(default_factory=list)

    def to_dict(self) -> dict:
        return {
            'name': self.name,
            'headers': [
                {'property_name': h.name, 'property_type': h.type.name.lower()}
                for h in self.headers
            ],
            'rows': self.rows,
        }


@dataclass
class Attr:
    """Scalar or list attribute result."""
    name: str
    type: PropertyType = PropertyType.UNSET
    values: List[Any] = field(default_factory=list)


"""Configuration for GQLDB Python driver."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import List, Optional
import ssl


@dataclass
class GqldbConfig:
    """Configuration for connecting to GQLDB."""

    # Server hosts in format "host:port"
    hosts: List[str] = field(default_factory=lambda: ["localhost:9000"])

    # Authentication credentials
    username: str = ""
    password: str = ""

    # Default graph to use
    default_graph: str = ""

    # Query timeout in seconds
    timeout: int = 30

    # Maximum receive message size in bytes (default 64MB)
    max_recv_size: int = 64 * 1024 * 1024

    # TLS/SSL configuration
    ssl_context: Optional[ssl.SSLContext] = None

    # Connection pool size per host
    pool_size: int = 10

    # Health check interval in seconds
    health_check_interval: float = 30.0

    # Number of retries for failed requests
    retry_count: int = 3

    # Delay between retries in seconds
    retry_delay: float = 0.1

    def validate(self) -> None:
        """Validate the configuration.

        Raises:
            ValueError: If configuration is invalid.
        """
        if not self.hosts:
            raise ValueError("No hosts configured")

        if self.timeout < 0:
            raise ValueError("Invalid timeout value")

        if self.max_recv_size <= 0:
            self.max_recv_size = 64 * 1024 * 1024  # Default to 64MB

        if self.pool_size <= 0:
            self.pool_size = 10


class ConfigBuilder:
    """Builder for creating GqldbConfig."""

    def __init__(self) -> None:
        self._config = GqldbConfig()

    def hosts(self, *hosts: str) -> "ConfigBuilder":
        """Set the server hosts."""
        self._config.hosts = list(hosts)
        return self

    def username(self, username: str) -> "ConfigBuilder":
        """Set the username for authentication."""
        self._config.username = username
        return self

    def password(self, password: str) -> "ConfigBuilder":
        """Set the password for authentication."""
        self._config.password = password
        return self

    def default_graph(self, graph: str) -> "ConfigBuilder":
        """Set the default graph."""
        self._config.default_graph = graph
        return self

    def timeout(self, seconds: int) -> "ConfigBuilder":
        """Set the query timeout in seconds."""
        self._config.timeout = seconds
        return self

    def max_recv_size(self, bytes_: int) -> "ConfigBuilder":
        """Set the maximum receive message size."""
        self._config.max_recv_size = bytes_
        return self

    def ssl(self, ssl_context: ssl.SSLContext) -> "ConfigBuilder":
        """Set the SSL context."""
        self._config.ssl_context = ssl_context
        return self

    def pool_size(self, size: int) -> "ConfigBuilder":
        """Set the connection pool size per host."""
        self._config.pool_size = size
        return self

    def health_check_interval(self, seconds: float) -> "ConfigBuilder":
        """Set the health check interval."""
        self._config.health_check_interval = seconds
        return self

    def retry_count(self, count: int) -> "ConfigBuilder":
        """Set the number of retries."""
        self._config.retry_count = count
        return self

    def retry_delay(self, seconds: float) -> "ConfigBuilder":
        """Set the delay between retries."""
        self._config.retry_delay = seconds
        return self

    def build(self) -> GqldbConfig:
        """Build and return the configuration."""
        self._config.validate()
        return self._config


def create_ssl_context(
    cert_file: Optional[str] = None,
    key_file: Optional[str] = None,
    ca_file: Optional[str] = None,
    verify: bool = True,
) -> ssl.SSLContext:
    """Create an SSL context for secure connections.

    Args:
        cert_file: Path to client certificate file.
        key_file: Path to client key file.
        ca_file: Path to CA certificate file.
        verify: Whether to verify server certificates.

    Returns:
        Configured SSL context.
    """
    context = ssl.create_default_context()

    if not verify:
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
    elif ca_file:
        context.load_verify_locations(ca_file)

    if cert_file and key_file:
        context.load_cert_chain(cert_file, key_file)

    return context
